@@ -77,9 +77,9 @@ struct permts {                 /* permanent text storage */
     char data[PERMTS_SIZE];     /* ... the data block itself */
 };
 
-extern bool global_offset_changed;       /* defined in nasm.c */
+extern int64_t global_offset_changed;       /* defined in nasm.c */
 
-static struct hash_table *ltab;		/* labels hash table */
+static struct hash_table ltab;		/* labels hash table */
 static union label *ldata;		/* all label data blocks */
 static union label *lfree;		/* labels free block */
 static struct permts *perm_head;        /* start of perm. text storage */
@@ -122,7 +122,7 @@ static union label *find_label(char *label, int create)
 	prevlen = 0;
     }
 
-    lpp = (union label **) hash_find(ltab, label, &ip);
+    lpp = (union label **) hash_find(&ltab, label, &ip);
     lptr = lpp ? *lpp : NULL;
 
     if (lptr || !create)
@@ -187,7 +187,6 @@ void redefine_label(char *label, int32_t segment, int64_t offset, char *special,
      * are even possible, nor whether they are checked somewhere else
      */
 
-    (void)segment;              /* Don't warn that this parameter is unused */
     (void)special;              /* Don't warn that this parameter is unused */
     (void)is_norm;              /* Don't warn that this parameter is unused */
     (void)isextrn;              /* Don't warn that this parameter is unused */
@@ -210,8 +209,9 @@ void redefine_label(char *label, int32_t segment, int64_t offset, char *special,
             prevlabel = lptr->defn.label;
     }
 
-    global_offset_changed |= (lptr->defn.offset != offset);
+    if (lptr->defn.offset != offset) global_offset_changed++;
     lptr->defn.offset = offset;
+    lptr->defn.segment = segment;
 
     if (pass0 == 1) {
         exi = !!(lptr->defn.is_global & GLOBAL_BIT);
@@ -281,7 +281,7 @@ void define_label(char *label, int32_t segment, int64_t offset, char *special,
     lptr->defn.offset = offset;
     lptr->defn.is_norm = (!islocalchar(label[0]) && is_norm);
 
-    if (pass0 == 1 || (!is_norm && !isextrn && (segment & 1))) {
+    if (pass0 == 1 || (!is_norm && !isextrn && (segment > 0) && (segment & 1))) {
         exi = !!(lptr->defn.is_global & GLOBAL_BIT);
         if (exi) {
             char *xsymbol;
@@ -371,7 +371,7 @@ void declare_as_global(char *label, char *special, efunc error)
 
 int init_labels(void)
 {
-    ltab = hash_init();
+    hash_init(&ltab, HASH_LARGE);
 
     ldata = lfree = (union label *)nasm_malloc(LBLK_SIZE);
     init_block(lfree);
@@ -396,7 +396,7 @@ void cleanup_labels(void)
 
     initialized = false;
 
-    hash_free(ltab);
+    hash_free(&ltab);
 
     lptr = lhold = ldata;
     while (lptr) {
@@ -443,6 +443,11 @@ static char *perm_copy(const char *string)
     return p;
 }
 
+char *local_scope(char *label)
+{
+   return islocal(label) ? prevlabel : "";
+}
+
 /*
  * Notes regarding bug involving redefinition of external segments.
  *
