@@ -25,6 +25,21 @@ efunc nasm_malloc_error;	/* Exported for the benefit of vsnprintf.c */
 static FILE *logfp;
 #endif
 
+/*
+ * Prepare a table of tolower() results.  This avoids function calls
+ * on some platforms.
+ */
+
+unsigned char nasm_tolower_tab[256];
+
+void tolower_init(void)
+{
+    int i;
+
+    for (i = 0; i < 256; i++)
+	nasm_tolower_tab[i] = tolower(i);
+}
+
 void nasm_set_malloc_error(efunc error)
 {
     nasm_malloc_error = error;
@@ -153,8 +168,8 @@ int nasm_stricmp(const char *s1, const char *s2)
     int d;
 
     while (1) {
-	c1 = *s1++;
-	c2 = *s2++;
+	c1 = nasm_tolower(*s1++);
+	c2 = nasm_tolower(*s2++);
 	d = c1-c2;
 
 	if (d)
@@ -173,8 +188,8 @@ int nasm_strnicmp(const char *s1, const char *s2, size_t n)
     int d;
 
     while (n--) {
-	c1 = *s1++;
-	c2 = *s2++;
+	c1 = nasm_tolower(*s1++);
+	c2 = nasm_tolower(*s2++);
 	d = c1-c2;
 
 	if (d)
@@ -192,8 +207,8 @@ int nasm_memicmp(const char *s1, const char *s2, size_t n)
     int d;
 
     while (n--) {
-	c1 = tolower(*s1++);
-	c2 = tolower(*s2++);
+	c1 = nasm_tolower(*s1++);
+	c2 = nasm_tolower(*s2++);
 	d = c1-c2;
 	if (d)
 	    return d;
@@ -220,7 +235,7 @@ char *nasm_strsep(char **stringp, const char *delim)
 #endif
 
 
-#define lib_isnumchar(c)   (isalnum(c) || (c) == '$' || (c) == '_')
+#define lib_isnumchar(c)   (nasm_isalnum(c) || (c) == '$' || (c) == '_')
 #define numvalue(c)  ((c)>='a' ? (c)-'a'+10 : (c)>='A' ? (c)-'A'+10 : (c)-'0')
 
 static int radix_letter(char c)
@@ -255,7 +270,7 @@ int64_t readnum(char *str, bool *error)
 
     *error = false;
 
-    while (isspace(*r))
+    while (nasm_isspace(*r))
         r++;                    /* find start of number */
 
     /*
@@ -385,7 +400,7 @@ int32_t seg_alloc(void)
     return (next_seg += 2) - 2;
 }
 
-#if X86_MEMORY
+#ifdef WORDS_LITTLEENDIAN
 
 void fwriteint16_t(uint16_t data, FILE * fp)
 {
@@ -407,7 +422,7 @@ void fwriteaddr(uint64_t data, int size, FILE * fp)
     fwrite(&data, 1, size, fp);
 }
 
-#else /* !X86_MEMORY */
+#else /* not WORDS_LITTLEENDIAN */
 
 void fwriteint16_t(uint16_t data, FILE * fp)
 {
@@ -469,404 +484,6 @@ void standard_extension(char *inname, char *outname, char *extension,
         strcpy(p, extension);
 }
 
-#define LEAFSIZ (sizeof(RAA)-sizeof(RAA_UNION)+sizeof(RAA_LEAF))
-#define BRANCHSIZ (sizeof(RAA)-sizeof(RAA_UNION)+sizeof(RAA_BRANCH))
-
-#define LAYERSHIFT(r) ( (r)->layers==0 ? RAA_BLKSHIFT : RAA_LAYERSHIFT )
-
-static struct RAA *real_raa_init(int layers)
-{
-    struct RAA *r;
-    int i;
-
-    if (layers == 0) {
-        r = nasm_zalloc(LEAFSIZ);
-        r->shift = 0;
-    } else {
-        r = nasm_malloc(BRANCHSIZ);
-        r->layers = layers;
-        for (i = 0; i < RAA_LAYERSIZE; i++)
-            r->u.b.data[i] = NULL;
-        r->shift = (RAA_BLKSHIFT-RAA_LAYERSHIFT) + layers*RAA_LAYERSHIFT;
-    }
-    return r;
-}
-
-struct RAA *raa_init(void)
-{
-    return real_raa_init(0);
-}
-
-void raa_free(struct RAA *r)
-{
-    if (r->layers) {
-        struct RAA **p;
-        for (p = r->u.b.data; p - r->u.b.data < RAA_LAYERSIZE; p++)
-            if (*p)
-                raa_free(*p);
-    }
-    nasm_free(r);
-}
-
-int64_t raa_read(struct RAA *r, int32_t posn)
-{
-    if ((uint32_t)posn >= (UINT32_C(1) << (r->shift + LAYERSHIFT(r))))
-        return 0;               /* Return 0 for undefined entries */
-    while (r->layers > 0) {
-	int32_t l = posn >> r->shift;
-	posn &= (UINT32_C(1) << r->shift)-1;
-        r = r->u.b.data[l];
-        if (!r)
-            return 0;           /* Return 0 for undefined entries */
-    }
-    return r->u.l.data[posn];
-}
-
-struct RAA *raa_write(struct RAA *r, int32_t posn, int64_t value)
-{
-    struct RAA *result;
-
-    if (posn < 0)
-        nasm_malloc_error(ERR_PANIC, "negative position in raa_write");
-
-    while ((UINT32_C(1) << (r->shift+LAYERSHIFT(r))) <= (uint32_t)posn) {
-        /*
-         * Must add a layer.
-         */
-        struct RAA *s;
-        int i;
-
-        s = nasm_malloc(BRANCHSIZ);
-        for (i = 0; i < RAA_LAYERSIZE; i++)
-            s->u.b.data[i] = NULL;
-        s->layers = r->layers + 1;
-        s->shift = LAYERSHIFT(r) + r->shift;
-        s->u.b.data[0] = r;
-        r = s;
-    }
-
-    result = r;
-
-    while (r->layers > 0) {
-        struct RAA **s;
-	int32_t l = posn >> r->shift;
-	posn &= (UINT32_C(1) << r->shift)-1;
-        s = &r->u.b.data[l];
-        if (!*s)
-            *s = real_raa_init(r->layers - 1);
-        r = *s;
-    }
-
-    r->u.l.data[posn] = value;
-
-    return result;
-}
-
-/* Aggregate SAA components smaller than this */
-#define SAA_BLKLEN 65536
-
-struct SAA *saa_init(size_t elem_len)
-{
-    struct SAA *s;
-    char *data;
-
-    s = nasm_zalloc(sizeof(struct SAA));
-
-    if (elem_len >= SAA_BLKLEN)
-	s->blk_len = elem_len;
-    else
-	s->blk_len = SAA_BLKLEN - (SAA_BLKLEN % elem_len);
-
-    s->elem_len = elem_len;
-    s->length = s->blk_len;
-    data = nasm_malloc(s->blk_len);
-    s->nblkptrs = s->nblks = 1;
-    s->blk_ptrs = nasm_malloc(sizeof(char *));
-    s->blk_ptrs[0] = data;
-    s->wblk = s->rblk = &s->blk_ptrs[0];
-
-    return s;
-}
-
-void saa_free(struct SAA *s)
-{
-    char **p;
-    size_t n;
-
-    for (p = s->blk_ptrs, n = s->nblks; n; p++, n--)
-	nasm_free(*p);
-
-    nasm_free(s->blk_ptrs);
-    nasm_free(s);
-}
-
-/* Add one allocation block to an SAA */
-static void saa_extend(struct SAA *s)
-{
-    size_t blkn = s->nblks++;
-
-    if (blkn >= s->nblkptrs) {
-	size_t rindex = s->rblk - s->blk_ptrs;
-	size_t windex = s->wblk - s->blk_ptrs;
-
-	s->nblkptrs <<= 1;
-	s->blk_ptrs = nasm_realloc(s->blk_ptrs, s->nblkptrs*sizeof(char *));
-
-	s->rblk = s->blk_ptrs + rindex;
-	s->wblk = s->blk_ptrs + windex;
-    }
-
-    s->blk_ptrs[blkn] = nasm_malloc(s->blk_len);
-    s->length += s->blk_len;
-}
-
-void *saa_wstruct(struct SAA *s)
-{
-    void *p;
-
-    if (s->wpos % s->elem_len)
-	    nasm_malloc_error(ERR_PANIC|ERR_NOFILE,
-			      "misaligned wpos in saa_wstruct");
-
-    if (s->wpos + s->elem_len > s->blk_len) {
-	if (s->wpos != s->blk_len)
-	    nasm_malloc_error(ERR_PANIC|ERR_NOFILE,
-			      "unfilled block in saa_wstruct");
-
-	if (s->wptr + s->elem_len > s->length)
-	    saa_extend(s);
-	s->wblk++;
-	s->wpos = 0;
-    }
-
-    p = *s->wblk + s->wpos;
-    s->wpos += s->elem_len;
-    s->wptr += s->elem_len;
-
-    if (s->wptr > s->datalen)
-	s->datalen = s->wptr;
-
-    return p;
-}
-
-void saa_wbytes(struct SAA *s, const void *data, size_t len)
-{
-    const char *d = data;
-
-    while (len) {
-        size_t l = s->blk_len - s->wpos;
-        if (l > len)
-            l = len;
-        if (l) {
-            if (d) {
-                memcpy(*s->wblk + s->wpos, d, l);
-                d += l;
-            } else
-                memset(*s->wblk + s->wpos, 0, l);
-            s->wpos += l;
-	    s->wptr += l;
-            len -= l;
-
-	    if (s->datalen < s->wptr)
-		s->datalen = s->wptr;
-        }
-        if (len) {
-	    if (s->wptr >= s->length)
-		saa_extend(s);
-	    s->wblk++;
-	    s->wpos = 0;
-	}
-    }
-}
-
-/* write unsigned LEB128 value to SAA */
-void saa_wleb128u(struct SAA *psaa, int value)
-{
-  char temp[64], *ptemp;
-  uint8_t byte;
-  int len;
-
-  ptemp = temp;
-  len = 0;
-  do
-  {
-     byte = value & 127;
-     value >>= 7;
-     if (value != 0) /* more bytes to come */
-        byte |= 0x80;
-     *ptemp = byte;
-     ptemp++;
-     len++;
-  } while (value != 0);
-  saa_wbytes(psaa, temp, len);
-}
-
-/* write signed LEB128 value to SAA */
-void saa_wleb128s(struct SAA *psaa, int value)
-{
-  char temp[64], *ptemp;
-  uint8_t byte;
-  bool more, negative;
-  int size, len;
-
-  ptemp = temp;
-  more = 1;
-  negative = (value < 0);
-  size = sizeof(int) * 8;
-  len = 0;
-  while(more)
-  {
-    byte = value & 0x7f;
-    value >>= 7;
-    if (negative)
-     /* sign extend */
-     value |= - (1 <<(size - 7));
-    /* sign bit of byte is second high order bit (0x40) */
-    if ((value == 0 && ! (byte & 0x40)) ||
-       ((value == -1) && (byte & 0x40)))
-       more = 0;
-    else
-      byte |= 0x80;
-    *ptemp = byte;
-    ptemp++;
-    len++;
-  }
-  saa_wbytes(psaa, temp, len);
-}
-
-void saa_rewind(struct SAA *s)
-{
-    s->rblk = s->blk_ptrs;
-    s->rpos = s->rptr = 0;
-}
-
-void *saa_rstruct(struct SAA *s)
-{
-    void *p;
-
-    if (s->rptr + s->elem_len > s->datalen)
-	return NULL;
-
-    if (s->rpos % s->elem_len)
-	    nasm_malloc_error(ERR_PANIC|ERR_NOFILE,
-			      "misaligned rpos in saa_rstruct");
-
-    if (s->rpos + s->elem_len > s->blk_len) {
-	s->rblk++;
-	s->rpos = 0;
-    }
-
-    p = *s->rblk + s->rpos;
-    s->rpos += s->elem_len;
-    s->rptr += s->elem_len;
-
-    return p;
-}
-
-const void *saa_rbytes(struct SAA *s, size_t *lenp)
-{
-    const void *p;
-    size_t len;
-
-    if (s->rptr >= s->datalen) {
-	*lenp = 0;
-	return NULL;
-    }
-
-    if (s->rpos >= s->blk_len) {
-	s->rblk++;
-	s->rpos = 0;
-    }
-
-    len = *lenp;
-    if (len > s->datalen - s->rptr)
-	len = s->datalen - s->rptr;
-    if (len > s->blk_len - s->rpos)
-	len = s->blk_len - s->rpos;
-
-    *lenp = len;
-    p = *s->rblk + s->rpos;
-
-    s->rpos += len;
-    s->rptr += len;
-
-    return p;
-}
-
-void saa_rnbytes(struct SAA *s, void *data, size_t len)
-{
-    char *d = data;
-
-    if (s->rptr + len > s->datalen) {
-	nasm_malloc_error(ERR_PANIC|ERR_NOFILE, "overrun in saa_rnbytes");
-	return;
-    }
-
-    while (len) {
-        size_t l;
-	const void *p;
-
-	l = len;
-	p = saa_rbytes(s, &l);
-
-	memcpy(d, p, l);
-	d   += l;
-	len -= l;
-    }
-}
-
-/* Same as saa_rnbytes, except position the counter first */
-void saa_fread(struct SAA *s, size_t posn, void *data, size_t len)
-{
-    size_t ix;
-
-    if (posn+len > s->datalen) {
-	nasm_malloc_error(ERR_PANIC|ERR_NOFILE, "overrun in saa_fread");
-	return;
-    }
-
-    ix = posn / s->blk_len;
-    s->rptr = posn;
-    s->rpos = posn % s->blk_len;
-    s->rblk = &s->blk_ptrs[ix];
-
-    saa_rnbytes(s, data, len);
-}
-
-/* Same as saa_wbytes, except position the counter first */
-void saa_fwrite(struct SAA *s, size_t posn, const void *data, size_t len)
-{
-    size_t ix;
-
-    if (posn > s->datalen) {
-	/* Seek beyond the end of the existing array not supported */
-	nasm_malloc_error(ERR_PANIC|ERR_NOFILE, "overrun in saa_fwrite");
-	return;
-    }
-
-    ix = posn / s->blk_len;
-    s->wptr = posn;
-    s->wpos = posn % s->blk_len;
-    s->wblk = &s->blk_ptrs[ix];
-
-    if (!s->wpos) {
-	s->wpos = s->blk_len;
-	s->wblk--;
-    }
-
-    saa_wbytes(s, data, len);
-}
-
-void saa_fpwrite(struct SAA *s, FILE * fp)
-{
-    const char *data;
-    size_t len;
-
-    saa_rewind(s);
-    while (len = s->datalen, (data = saa_rbytes(s, &len)) != NULL)
-        fwrite(data, 1, len, fp);
-}
-
 /*
  * Common list of prefix names
  */
