@@ -18,6 +18,23 @@
 #include <strings.h>
 #endif
 
+/*
+ * tolower table -- avoids a function call on some platforms.
+ * NOTE: unlike the tolower() function in ctype, EOF is *NOT*
+ * a permitted value, for obvious reasons.
+ */
+void tolower_init(void);
+extern unsigned char nasm_tolower_tab[256];
+#define nasm_tolower(x) nasm_tolower_tab[(unsigned char)(x)]
+
+/* Wrappers around <ctype.h> functions */
+/* These are only valid for values that cannot include EOF */
+#define nasm_isspace(x)  isspace((unsigned char)(x))
+#define nasm_isalpha(x)  isalpha((unsigned char)(x))
+#define nasm_isdigit(x)  isdigit((unsigned char)(x))
+#define nasm_isalnum(x)  isalnum((unsigned char)(x))
+#define nasm_isxdigit(x) isxdigit((unsigned char)(x))
+
 /*
  * If this is defined, the wrappers around malloc et al will
  * transform into logging variants, which will cause NASM to create
@@ -37,6 +54,7 @@
  * An error reporting function should look like this.
  */
 typedef void (*efunc) (int severity, const char *fmt, ...);
+extern efunc nasm_malloc_error;
 
 /*
  * These are the error severity codes which get passed as the first
@@ -53,27 +71,31 @@ typedef void (*efunc) (int severity, const char *fmt, ...);
 #define ERR_NOFILE	0x00000010      /* don't give source file name/line */
 #define ERR_USAGE	0x00000020      /* print a usage message */
 #define ERR_PASS1	0x00000040      /* only print this error on pass one */
+#define ERR_PASS2	0x00000080
+#define ERR_NO_SEVERITY	0x00000100      /* suppress printing severity */
 
 /*
  * These codes define specific types of suppressible warning.
  */
 
-#define ERR_WARN_MASK	0x0000FF00      /* the mask for this feature */
-#define ERR_WARN_SHR  8			/* how far to shift right */
+#define ERR_WARN_MASK	0xFFFFF000      /* the mask for this feature */
+#define ERR_WARN_SHR 	12               /* how far to shift right */
 
 #define WARN(x) ((x) << ERR_WARN_SHR)
 
-#define ERR_WARN_MNP		WARN(1) /* macro-num-parameters warning */
-#define ERR_WARN_MSR		WARN(2) /* macro self-reference */
-#define ERR_WARN_OL		WARN(3)	/* orphan label (no colon, and
-                                         * alone on line) */
-#define ERR_WARN_NOV		WARN(4)	/* numeric overflow */
-#define ERR_WARN_GNUELF		WARN(5)	/* using GNU ELF extensions */
-#define ERR_WARN_FL_OVERFLOW	WARN(6) /* FP overflow */
-#define ERR_WARN_FL_DENORM	WARN(7) /* FP denormal */
-#define ERR_WARN_FL_UNDERFLOW	WARN(8)	/* FP underflow */
-#define ERR_WARN_FL_TOOLONG	WARN(9) /* FP too many digits */
-#define ERR_WARN_MAX	9		/* the highest numbered one */
+#define ERR_WARN_MNP		WARN( 1) /* macro-num-parameters warning */
+#define ERR_WARN_MSR		WARN( 2) /* macro self-reference */
+#define ERR_WARN_MDP		WARN( 3) /* macro default parameters check */
+#define ERR_WARN_OL 		WARN( 4) /* orphan label (no colon, and
+                                          * alone on line) */
+#define ERR_WARN_NOV		WARN( 5) /* numeric overflow */
+#define ERR_WARN_GNUELF		WARN( 6) /* using GNU ELF extensions */
+#define ERR_WARN_FL_OVERFLOW	WARN( 7) /* FP overflow */
+#define ERR_WARN_FL_DENORM	WARN( 8) /* FP denormal */
+#define ERR_WARN_FL_UNDERFLOW	WARN( 9) /* FP underflow */
+#define ERR_WARN_FL_TOOLONG	WARN(10) /* FP too many digits */
+#define ERR_WARN_USER		WARN(11) /* %warning directives */
+#define ERR_WARN_MAX		11	 /* the highest numbered one */
 
 /*
  * Wrappers around malloc, realloc and free. nasm_malloc will
@@ -173,7 +195,6 @@ void standard_extension(char *inname, char *outname, char *extension,
 
 #define elements(x)     ( sizeof(x) / sizeof(*(x)) )
 
-
 /*
  * some handy macros that will probably be of use in more than one
  * output format: convert integers into little-endian byte packed
@@ -279,103 +300,6 @@ void fwriteint32_t(uint32_t data, FILE * fp);
 void fwriteint64_t(uint64_t data, FILE * fp);
 void fwriteaddr(uint64_t data, int size, FILE * fp);
 
-/*
- * Routines to manage a dynamic random access array of int64_ts which
- * may grow in size to be more than the largest single malloc'able
- * chunk.
- */
-
-#define RAA_BLKSHIFT	15	/* 2**this many longs allocated at once */
-#define RAA_BLKSIZE	(1 << RAA_BLKSHIFT)
-#define RAA_LAYERSHIFT	15	/* 2**this many _pointers_ allocated */
-#define RAA_LAYERSIZE	(1 << RAA_LAYERSHIFT)
-
-typedef struct RAA RAA;
-typedef union RAA_UNION RAA_UNION;
-typedef struct RAA_LEAF RAA_LEAF;
-typedef struct RAA_BRANCH RAA_BRANCH;
-
-struct RAA {
-    /*
-     * Number of layers below this one to get to the real data. 0
-     * means this structure is a leaf, holding RAA_BLKSIZE real
-     * data items; 1 and above mean it's a branch, holding
-     * RAA_LAYERSIZE pointers to the next level branch or leaf
-     * structures.
-     */
-    int layers;
-
-    /*
-     * Number of real data items spanned by one position in the
-     * `data' array at this level. This number is 0 trivially, for
-     * a leaf (level 0): for a level 1 branch it should be
-     * RAA_BLKSHIFT, and for a level 2 branch it's
-     * RAA_LAYERSHIFT+RAA_BLKSHIFT.
-     */
-    int shift;
-
-    union RAA_UNION {
-        struct RAA_LEAF {
-            int64_t data[RAA_BLKSIZE];
-        } l;
-        struct RAA_BRANCH {
-            struct RAA *data[RAA_LAYERSIZE];
-        } b;
-    } u;
-};
-
-struct RAA *raa_init(void);
-void raa_free(struct RAA *);
-int64_t raa_read(struct RAA *, int32_t);
-struct RAA *raa_write(struct RAA *r, int32_t posn, int64_t value);
-
-/*
- * Routines to manage a dynamic sequential-access array, under the
- * same restriction on maximum mallocable block. This array may be
- * written to in two ways: a contiguous chunk can be reserved of a
- * given size with a pointer returned OR single-byte data may be
- * written. The array can also be read back in the same two ways:
- * as a series of big byte-data blocks or as a list of structures
- * of a given size.
- */
-
-struct SAA {
-    /*
-     * members `end' and `elem_len' are only valid in first link in
-     * list; `rptr' and `rpos' are used for reading
-     */
-    size_t elem_len;		/* Size of each element */
-    size_t blk_len;		/* Size of each allocation block */
-    size_t nblks;		/* Total number of allocated blocks */
-    size_t nblkptrs;		/* Total number of allocation block pointers */
-    size_t length;		/* Total allocated length of the array */
-    size_t datalen;		/* Total data length of the array */
-    char **wblk;		/* Write block pointer */
-    size_t wpos;		/* Write position inside block */
-    size_t wptr;		/* Absolute write position */
-    char **rblk;		/* Read block pointer */
-    size_t rpos;		/* Read position inside block */
-    size_t rptr;		/* Absolute read position */
-    char **blk_ptrs;		/* Pointer to pointer blocks */
-};
-
-struct SAA *saa_init(size_t elem_len);    /* 1 == byte */
-void saa_free(struct SAA *);
-void *saa_wstruct(struct SAA *);        /* return a structure of elem_len */
-void saa_wbytes(struct SAA *, const void *, size_t);      /* write arbitrary bytes */
-void saa_wleb128u(struct SAA *, int); /* write unsigned LEB128 value */
-void saa_wleb128s(struct SAA *, int); /* write signed LEB128 value */
-void saa_rewind(struct SAA *);  /* for reading from beginning */
-void *saa_rstruct(struct SAA *);        /* return NULL on EOA */
-const void *saa_rbytes(struct SAA *, size_t *); /* return 0 on EOA */
-void saa_rnbytes(struct SAA *, void *, size_t);   /* read a given no. of bytes */
-/* random access */
-void saa_fread(struct SAA *, size_t, void *, size_t);
-void saa_fwrite(struct SAA *, size_t, const void *, size_t);
-
-/* dump to file */
-void saa_fpwrite(struct SAA *, FILE *);
-
 /*
  * Binary search routine. Returns index into `array' of an entry
  * matching `string', or <0 if no match. `array' is taken to
