@@ -3,17 +3,23 @@
  *
  * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
  * Julian Hall. All rights reserved. The software is
- * redistributable under the licence given in the file "Licence"
+ * redistributable under the license given in the file "LICENSE"
  * distributed in the NASM archive.
  */
 
+#include "compiler.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "saa.h"
+#include "raa.h"
+#include "stdscan.h"
 #include "outform.h"
 
 #if defined OF_AOUT || defined OF_AOUTB
@@ -28,22 +34,22 @@
 
 struct Reloc {
     struct Reloc *next;
-    long address;               /* relative to _start_ of section */
-    long symbol;                /* symbol number or -ve section id */
+    int32_t address;               /* relative to _start_ of section */
+    int32_t symbol;                /* symbol number or -ve section id */
     int bytes;                  /* 2 or 4 */
     int reltype;                /* see above */
 };
 
 struct Symbol {
-    long strpos;                /* string table position of name */
+    int32_t strpos;                /* string table position of name */
     int type;                   /* symbol type - see flags below */
-    long value;                 /* address, or COMMON variable size */
-    long size;                  /* size for data or function exports */
-    long segment;               /* back-reference used by gsym_reloc */
+    int32_t value;                 /* address, or COMMON variable size */
+    int32_t size;                  /* size for data or function exports */
+    int32_t segment;               /* back-reference used by gsym_reloc */
     struct Symbol *next;        /* list of globals in each section */
     struct Symbol *nextfwd;     /* list of unresolved-size symbols */
     char *name;                 /* for unresolved-size symbols */
-    long symnum;                /* index into symbol table */
+    int32_t symnum;                /* index into symbol table */
 };
 
 /*
@@ -75,8 +81,8 @@ struct Symbol {
 
 struct Section {
     struct SAA *data;
-    unsigned long len, size, nrelocs;
-    long index;
+    uint32_t len, size, nrelocs;
+    int32_t index;
     struct Reloc *head, **tail;
     struct Symbol *gsyms, *asym;
 };
@@ -84,12 +90,12 @@ struct Section {
 static struct Section stext, sdata, sbss;
 
 static struct SAA *syms;
-static unsigned long nsyms;
+static uint32_t nsyms;
 
 static struct RAA *bsym;
 
 static struct SAA *strs;
-static unsigned long strslen;
+static uint32_t strslen;
 
 static struct Symbol *fwds;
 
@@ -103,8 +109,8 @@ static int is_pic;
 static void aout_write(void);
 static void aout_write_relocs(struct Reloc *);
 static void aout_write_syms(void);
-static void aout_sect_write(struct Section *, const unsigned char *,
-                            unsigned long);
+static void aout_sect_write(struct Section *, const uint8_t *,
+                            uint32_t);
 static void aout_pad_sections(void);
 static void aout_fixup_relocs(struct Section *);
 
@@ -113,9 +119,9 @@ static void aout_fixup_relocs(struct Section *);
  * symbols, which can be used with WRT to provide PIC relocation
  * types.
  */
-static long aout_gotpc_sect, aout_gotoff_sect;
-static long aout_got_sect, aout_plt_sect;
-static long aout_sym_sect;
+static int32_t aout_gotpc_sect, aout_gotoff_sect;
+static int32_t aout_got_sect, aout_plt_sect;
+static int32_t aout_sym_sect;
 
 static void aoutg_init(FILE * fp, efunc errfunc, ldfunc ldef,
                        evalfunc eval)
@@ -137,7 +143,7 @@ static void aoutg_init(FILE * fp, efunc errfunc, ldfunc ldef,
     sdata.index = seg_alloc();
     sbss.index = seg_alloc();
     stext.asym = sdata.asym = sbss.asym = NULL;
-    syms = saa_init((long)sizeof(struct Symbol));
+    syms = saa_init((int32_t)sizeof(struct Symbol));
     nsyms = 0;
     bsym = raa_init();
     strs = saa_init(1L);
@@ -149,7 +155,7 @@ static void aoutg_init(FILE * fp, efunc errfunc, ldfunc ldef,
 
 static void aout_init(FILE * fp, efunc errfunc, ldfunc ldef, evalfunc eval)
 {
-    bsd = FALSE;
+    bsd = false;
     aoutg_init(fp, errfunc, ldef, eval);
 
     aout_gotpc_sect = aout_gotoff_sect = aout_got_sect =
@@ -165,25 +171,25 @@ extern struct ofmt of_aoutb;
 static void aoutb_init(FILE * fp, efunc errfunc, ldfunc ldef,
                        evalfunc eval)
 {
-    bsd = TRUE;
+    bsd = true;
     aoutg_init(fp, errfunc, ldef, eval);
 
     is_pic = 0x00;              /* may become 0x40 */
 
     aout_gotpc_sect = seg_alloc();
-    ldef("..gotpc", aout_gotpc_sect + 1, 0L, NULL, FALSE, FALSE, &of_aoutb,
+    ldef("..gotpc", aout_gotpc_sect + 1, 0L, NULL, false, false, &of_aoutb,
          error);
     aout_gotoff_sect = seg_alloc();
-    ldef("..gotoff", aout_gotoff_sect + 1, 0L, NULL, FALSE, FALSE,
+    ldef("..gotoff", aout_gotoff_sect + 1, 0L, NULL, false, false,
          &of_aoutb, error);
     aout_got_sect = seg_alloc();
-    ldef("..got", aout_got_sect + 1, 0L, NULL, FALSE, FALSE, &of_aoutb,
+    ldef("..got", aout_got_sect + 1, 0L, NULL, false, false, &of_aoutb,
          error);
     aout_plt_sect = seg_alloc();
-    ldef("..plt", aout_plt_sect + 1, 0L, NULL, FALSE, FALSE, &of_aoutb,
+    ldef("..plt", aout_plt_sect + 1, 0L, NULL, false, false, &of_aoutb,
          error);
     aout_sym_sect = seg_alloc();
-    ldef("..sym", aout_sym_sect + 1, 0L, NULL, FALSE, FALSE, &of_aoutb,
+    ldef("..sym", aout_sym_sect + 1, 0L, NULL, false, false, &of_aoutb,
          error);
 }
 
@@ -217,8 +223,11 @@ static void aout_cleanup(int debuginfo)
     saa_free(strs);
 }
 
-static long aout_section_names(char *name, int pass, int *bits)
+static int32_t aout_section_names(char *name, int pass, int *bits)
 {
+
+    (void)pass;
+
     /*
      * Default to 32 bits.
      */
@@ -238,12 +247,12 @@ static long aout_section_names(char *name, int pass, int *bits)
         return NO_SEG;
 }
 
-static void aout_deflabel(char *name, long segment, long offset,
+static void aout_deflabel(char *name, int32_t segment, int64_t offset,
                           int is_global, char *special)
 {
     int pos = strslen + 4;
     struct Symbol *sym;
-    int special_used = FALSE;
+    int special_used = false;
 
     if (name[0] == '.' && name[1] == '.' && name[2] != '@') {
         /*
@@ -270,9 +279,9 @@ static void aout_deflabel(char *name, long segment, long offset,
                 expr *e;
                 char *p = special;
 
-                while (*p && !isspace(*p))
+                while (*p && !nasm_isspace(*p))
                     p++;
-                while (*p && isspace(*p))
+                while (*p && nasm_isspace(*p))
                     p++;
                 stdscan_reset();
                 stdscan_bufptr = p;
@@ -296,7 +305,7 @@ static void aout_deflabel(char *name, long segment, long offset,
         return;                 /* it wasn't an important one */
     }
 
-    saa_wbytes(strs, name, (long)(1 + strlen(name)));
+    saa_wbytes(strs, name, (int32_t)(1 + strlen(name)));
     strslen += 1 + strlen(name);
 
     sym = saa_wstruct(syms);
@@ -354,14 +363,14 @@ static void aout_deflabel(char *name, long segment, long offset,
             if (special[n]) {
                 struct tokenval tokval;
                 expr *e;
-                int fwd = FALSE;
+                int fwd = false;
                 char *saveme = stdscan_bufptr;  /* bugfix? fbk 8/10/00 */
 
                 if (!bsd) {
                     error(ERR_NONFATAL, "Linux a.out does not support"
                           " symbol size information");
                 } else {
-                    while (special[n] && isspace(special[n]))
+                    while (special[n] && nasm_isspace(special[n]))
                         n++;
                     /*
                      * We have a size expression; attempt to
@@ -387,7 +396,7 @@ static void aout_deflabel(char *name, long segment, long offset,
                 }
                 stdscan_bufptr = saveme;        /* bugfix? fbk 8/10/00 */
             }
-            special_used = TRUE;
+            special_used = true;
         }
     }
 
@@ -408,7 +417,7 @@ static void aout_deflabel(char *name, long segment, long offset,
         error(ERR_NONFATAL, "no special symbol features supported here");
 }
 
-static void aout_add_reloc(struct Section *sect, long segment,
+static void aout_add_reloc(struct Section *sect, int32_t segment,
                            int reltype, int bytes)
 {
     struct Reloc *r;
@@ -453,8 +462,8 @@ static void aout_add_reloc(struct Section *sect, long segment,
  * Inefficiency: we search, currently, using a linked list which
  * isn't even necessarily sorted.
  */
-static long aout_add_gsym_reloc(struct Section *sect,
-                                long segment, long offset,
+static int32_t aout_add_gsym_reloc(struct Section *sect,
+                                int32_t segment, int32_t offset,
                                 int type, int bytes, int exact)
 {
     struct Symbol *sym, *sm, *shead;
@@ -526,8 +535,8 @@ static long aout_add_gsym_reloc(struct Section *sect,
  * Return value is the adjusted value of `addr', having become an
  * offset from the `asym' symbol rather than the section.
  */
-static long aout_add_gotoff_reloc(struct Section *sect, long segment,
-                                  long offset, int bytes)
+static int32_t aout_add_gotoff_reloc(struct Section *sect, int32_t segment,
+                                  int32_t offset, int bytes)
 {
     struct Reloc *r;
     struct Symbol *asym;
@@ -561,15 +570,13 @@ static long aout_add_gotoff_reloc(struct Section *sect, long segment,
     return offset - asym->value;
 }
 
-static void aout_out(long segto, const void *data, unsigned long type,
-                     long segment, long wrt)
+static void aout_out(int32_t segto, const void *data,
+		     enum out_type type, uint64_t size,
+                     int32_t segment, int32_t wrt)
 {
     struct Section *s;
-    long realbytes = type & OUT_SIZMASK;
-    long addr;
-    unsigned char mydata[4], *p;
-
-    type &= OUT_TYPMASK;
+    int32_t addr;
+    uint8_t mydata[4], *p;
 
     /*
      * handle absolute-assembly (structure definitions)
@@ -594,30 +601,30 @@ static void aout_out(long segto, const void *data, unsigned long type,
     }
 
     if (!s && type != OUT_RESERVE) {
-        error(ERR_WARNING, "attempt to initialise memory in the"
+        error(ERR_WARNING, "attempt to initialize memory in the"
               " BSS section: ignored");
         if (type == OUT_REL2ADR)
-            realbytes = 2;
+            size = 2;
         else if (type == OUT_REL4ADR)
-            realbytes = 4;
-        sbss.len += realbytes;
+            size = 4;
+        sbss.len += size;
         return;
     }
 
     if (type == OUT_RESERVE) {
         if (s) {
-            error(ERR_WARNING, "uninitialised space declared in"
+            error(ERR_WARNING, "uninitialized space declared in"
                   " %s section: zeroing",
                   (segto == stext.index ? "code" : "data"));
-            aout_sect_write(s, NULL, realbytes);
+            aout_sect_write(s, NULL, size);
         } else
-            sbss.len += realbytes;
+            sbss.len += size;
     } else if (type == OUT_RAWDATA) {
         if (segment != NO_SEG)
             error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
-        aout_sect_write(s, data, realbytes);
+        aout_sect_write(s, data, size);
     } else if (type == OUT_ADDRESS) {
-        addr = *(long *)data;
+        addr = *(int64_t *)data;
         if (segment != NO_SEG) {
             if (segment % 2) {
                 error(ERR_NONFATAL, "a.out format does not support"
@@ -625,7 +632,7 @@ static void aout_out(long segto, const void *data, unsigned long type,
             } else {
                 if (wrt == NO_SEG) {
                     aout_add_reloc(s, segment, RELTYPE_ABSOLUTE,
-                                   realbytes);
+                                   size);
                 } else if (!bsd) {
                     error(ERR_NONFATAL,
                           "Linux a.out format does not support"
@@ -633,20 +640,20 @@ static void aout_out(long segto, const void *data, unsigned long type,
                     wrt = NO_SEG;       /* we can at least _try_ to continue */
                 } else if (wrt == aout_gotpc_sect + 1) {
                     is_pic = 0x40;
-                    aout_add_reloc(s, segment, RELTYPE_GOTPC, realbytes);
+                    aout_add_reloc(s, segment, RELTYPE_GOTPC, size);
                 } else if (wrt == aout_gotoff_sect + 1) {
                     is_pic = 0x40;
                     addr = aout_add_gotoff_reloc(s, segment,
-                                                 addr, realbytes);
+                                                 addr, size);
                 } else if (wrt == aout_got_sect + 1) {
                     is_pic = 0x40;
                     addr =
                         aout_add_gsym_reloc(s, segment, addr, RELTYPE_GOT,
-                                            realbytes, TRUE);
+                                            size, true);
                 } else if (wrt == aout_sym_sect + 1) {
                     addr = aout_add_gsym_reloc(s, segment, addr,
-                                               RELTYPE_ABSOLUTE, realbytes,
-                                               FALSE);
+                                               RELTYPE_ABSOLUTE, size,
+                                               false);
                 } else if (wrt == aout_plt_sect + 1) {
                     is_pic = 0x40;
                     error(ERR_NONFATAL,
@@ -661,11 +668,11 @@ static void aout_out(long segto, const void *data, unsigned long type,
             }
         }
         p = mydata;
-        if (realbytes == 2)
+        if (size == 2)
             WRITESHORT(p, addr);
         else
             WRITELONG(p, addr);
-        aout_sect_write(s, mydata, realbytes);
+        aout_sect_write(s, mydata, size);
     } else if (type == OUT_REL2ADR) {
         if (segment == segto)
             error(ERR_PANIC, "intra-segment OUT_REL2ADR");
@@ -694,7 +701,7 @@ static void aout_out(long segto, const void *data, unsigned long type,
             }
         }
         p = mydata;
-        WRITESHORT(p, *(long *)data - (realbytes + s->len));
+        WRITESHORT(p, *(int64_t *)data - (size + s->len));
         aout_sect_write(s, mydata, 2L);
     } else if (type == OUT_REL4ADR) {
         if (segment == segto)
@@ -724,21 +731,21 @@ static void aout_out(long segto, const void *data, unsigned long type,
             }
         }
         p = mydata;
-        WRITELONG(p, *(long *)data - (realbytes + s->len));
+        WRITELONG(p, *(int64_t *)data - (size + s->len));
         aout_sect_write(s, mydata, 4L);
     }
 }
 
 static void aout_pad_sections(void)
 {
-    static unsigned char pad[] = { 0x90, 0x90, 0x90, 0x90 };
+    static uint8_t pad[] = { 0x90, 0x90, 0x90, 0x90 };
     /*
      * Pad each of the text and data sections with NOPs until their
      * length is a multiple of four. (NOP == 0x90.) Also increase
      * the length of the BSS section similarly.
      */
-    aout_sect_write(&stext, pad, (-(long)stext.len) & 3);
-    aout_sect_write(&sdata, pad, (-(long)sdata.len) & 3);
+    aout_sect_write(&stext, pad, (-(int32_t)stext.len) & 3);
+    aout_sect_write(&sdata, pad, (-(int32_t)sdata.len) & 3);
     sbss.len = (sbss.len + 3) & ~3;
 }
 
@@ -757,17 +764,17 @@ static void aout_fixup_relocs(struct Section *sect)
 
     saa_rewind(sect->data);
     for (r = sect->head; r; r = r->next) {
-        unsigned char *p, *q, blk[4];
-        long l;
+        uint8_t *p, *q, blk[4];
+        int32_t l;
 
-        saa_fread(sect->data, r->address, blk, (long)r->bytes);
+        saa_fread(sect->data, r->address, blk, (int32_t)r->bytes);
         p = q = blk;
         l = *p++;
         if (r->bytes > 1) {
-            l += ((long)*p++) << 8;
+            l += ((int32_t)*p++) << 8;
             if (r->bytes == 4) {
-                l += ((long)*p++) << 16;
-                l += ((long)*p++) << 24;
+                l += ((int32_t)*p++) << 16;
+                l += ((int32_t)*p++) << 24;
             }
         }
         if (r->symbol == -SECT_DATA)
@@ -780,7 +787,7 @@ static void aout_fixup_relocs(struct Section *sect)
             WRITESHORT(q, l);
         else
             *q++ = l & 0xFF;
-        saa_fwrite(sect->data, r->address, blk, (long)r->bytes);
+        saa_fwrite(sect->data, r->address, blk, (int32_t)r->bytes);
     }
 }
 
@@ -790,14 +797,14 @@ static void aout_write(void)
      * Emit the a.out header.
      */
     /* OMAGIC, M_386 or MID_I386, no flags */
-    fwritelong(bsd ? 0x07018600 | is_pic : 0x640107L, aoutfp);
-    fwritelong(stext.len, aoutfp);
-    fwritelong(sdata.len, aoutfp);
-    fwritelong(sbss.len, aoutfp);
-    fwritelong(nsyms * 12, aoutfp);     /* length of symbol table */
-    fwritelong(0L, aoutfp);     /* object files have no entry point */
-    fwritelong(stext.nrelocs * 8, aoutfp);      /* size of text relocs */
-    fwritelong(sdata.nrelocs * 8, aoutfp);      /* size of data relocs */
+    fwriteint32_t(bsd ? 0x07018600 | is_pic : 0x640107L, aoutfp);
+    fwriteint32_t(stext.len, aoutfp);
+    fwriteint32_t(sdata.len, aoutfp);
+    fwriteint32_t(sbss.len, aoutfp);
+    fwriteint32_t(nsyms * 12, aoutfp);     /* length of symbol table */
+    fwriteint32_t(0L, aoutfp);     /* object files have no entry point */
+    fwriteint32_t(stext.nrelocs * 8, aoutfp);      /* size of text relocs */
+    fwriteint32_t(sdata.nrelocs * 8, aoutfp);      /* size of data relocs */
 
     /*
      * Write out the code section and the data section.
@@ -819,16 +826,16 @@ static void aout_write(void)
     /*
      * And the string table.
      */
-    fwritelong(strslen + 4, aoutfp);    /* length includes length count */
+    fwriteint32_t(strslen + 4, aoutfp);    /* length includes length count */
     saa_fpwrite(strs, aoutfp);
 }
 
 static void aout_write_relocs(struct Reloc *r)
 {
     while (r) {
-        unsigned long word2;
+        uint32_t word2;
 
-        fwritelong(r->address, aoutfp);
+        fwriteint32_t(r->address, aoutfp);
 
         if (r->symbol >= 0)
             word2 = r->symbol;
@@ -837,7 +844,7 @@ static void aout_write_relocs(struct Reloc *r)
         word2 |= r->reltype << 24;
         word2 |= (r->bytes == 1 ? 0 :
                   r->bytes == 2 ? 0x2000000L : 0x4000000L);
-        fwritelong(word2, aoutfp);
+        fwriteint32_t(word2, aoutfp);
 
         r = r->next;
     }
@@ -845,13 +852,13 @@ static void aout_write_relocs(struct Reloc *r)
 
 static void aout_write_syms(void)
 {
-    unsigned long i;
+    uint32_t i;
 
     saa_rewind(syms);
     for (i = 0; i < nsyms; i++) {
         struct Symbol *sym = saa_rstruct(syms);
-        fwritelong(sym->strpos, aoutfp);
-        fwritelong((long)sym->type & ~SYM_WITH_SIZE, aoutfp);
+        fwriteint32_t(sym->strpos, aoutfp);
+        fwriteint32_t((int32_t)sym->type & ~SYM_WITH_SIZE, aoutfp);
         /*
          * Fix up the symbol value now we know the final section
          * sizes.
@@ -860,33 +867,36 @@ static void aout_write_syms(void)
             sym->value += stext.len;
         if ((sym->type & SECT_MASK) == SECT_BSS)
             sym->value += stext.len + sdata.len;
-        fwritelong(sym->value, aoutfp);
+        fwriteint32_t(sym->value, aoutfp);
         /*
          * Output a size record if necessary.
          */
         if (sym->type & SYM_WITH_SIZE) {
-            fwritelong(sym->strpos, aoutfp);
-            fwritelong(0x0DL, aoutfp);  /* special value: means size */
-            fwritelong(sym->size, aoutfp);
+            fwriteint32_t(sym->strpos, aoutfp);
+            fwriteint32_t(0x0DL, aoutfp);  /* special value: means size */
+            fwriteint32_t(sym->size, aoutfp);
             i++;                /* use up another of `nsyms' */
         }
     }
 }
 
 static void aout_sect_write(struct Section *sect,
-                            const unsigned char *data, unsigned long len)
+                            const uint8_t *data, uint32_t len)
 {
     saa_wbytes(sect->data, data, len);
     sect->len += len;
 }
 
-static long aout_segbase(long segment)
+static int32_t aout_segbase(int32_t segment)
 {
     return segment;
 }
 
 static int aout_directive(char *directive, char *value, int pass)
 {
+    (void)directive;
+    (void)value;
+    (void)pass;
     return 0;
 }
 
@@ -895,15 +905,12 @@ static void aout_filename(char *inname, char *outname, efunc error)
     standard_extension(inname, outname, ".o", error);
 }
 
-static const char *aout_stdmac[] = {
-    "%define __SECT__ [section .text]",
-    "%macro __NASM_CDecl__ 1",
-    "%endmacro",
-    NULL
-};
+extern macros_t aout_stdmac[];
 
 static int aout_set_info(enum geninfo type, char **val)
 {
+    (void)type;
+    (void)val;
     return 0;
 }
 #endif                          /* OF_AOUT || OF_AOUTB */
