@@ -3,7 +3,7 @@
  *
  * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
  * Julian Hall. All rights reserved. The software is
- * redistributable under the licence given in the file "Licence"
+ * redistributable under the license given in the file "LICENSE"
  * distributed in the NASM archive.
  */
 
@@ -44,13 +44,18 @@
 
 */
 
+#include "compiler.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "saa.h"
+#include "stdscan.h"
 #include "labels.h"
 #include "eval.h"
 #include "outform.h"
@@ -82,18 +87,18 @@ static struct bin_label {
 static struct Section {
     char *name;
     struct SAA *contents;
-    long length;                /* section length in bytes */
+    int64_t length;                /* section length in bytes */
 
 /* Section attributes */
     int flags;                  /* see flag definitions above */
-    unsigned long align;        /* section alignment */
-    unsigned long valign;       /* notional section alignment */
-    unsigned long start;        /* section start address */
-    unsigned long vstart;       /* section virtual start address */
+    uint64_t align;        /* section alignment */
+    uint64_t valign;       /* notional section alignment */
+    uint64_t start;        /* section start address */
+    uint64_t vstart;       /* section virtual start address */
     char *follows;              /* the section that this one will follow */
     char *vfollows;             /* the section that this one will notionally follow */
-    long start_index;           /* NASM section id for non-relocated version */
-    long vstart_index;          /* the NASM section id */
+    int32_t start_index;           /* NASM section id for non-relocated version */
+    int32_t vstart_index;          /* the NASM section id */
 
     struct bin_label *labels;   /* linked-list of label handles for map output. */
     struct bin_label **labels_end;      /* Holds address of end of labels list. */
@@ -112,19 +117,18 @@ static struct Section {
 
 static struct Reloc {
     struct Reloc *next;
-    long posn;
-    long bytes;
-    long secref;
-    long secrel;
+    int32_t posn;
+    int32_t bytes;
+    int32_t secref;
+    int32_t secrel;
     struct Section *target;
 } *relocs, **reloctail;
 
 extern char *stdscan_bufptr;
-extern int lookup_label(char *label, long *segment, long *offset);
 
-static unsigned char format_mode;       /* 0 = original bin, 1 = extended bin */
-static long current_section;    /* only really needed if format_mode = 0 */
-static unsigned long origin;
+static uint8_t format_mode;       /* 0 = original bin, 1 = extended bin */
+static int32_t current_section;    /* only really needed if format_mode = 0 */
+static uint64_t origin;
 static int origin_defined;
 
 /* Stuff we need for map-file generation. */
@@ -135,18 +139,10 @@ static int origin_defined;
 static int map_control = 0;
 static char *infile, *outfile;
 
-static const char *bin_stdmac[] = {
-    "%define __SECT__ [section .text]",
-    "%imacro org 1+.nolist",
-    "[org %1]",
-    "%endmacro",
-    "%macro __NASM_CDecl__ 1",
-    "%endmacro",
-    NULL
-};
+extern macros_t bin_stdmac[];
 
-static void add_reloc(struct Section *s, long bytes, long secref,
-                      long secrel)
+static void add_reloc(struct Section *s, int32_t bytes, int32_t secref,
+                      int32_t secrel)
 {
     struct Reloc *r;
 
@@ -170,7 +166,7 @@ static struct Section *find_section_by_name(const char *name)
     return s;
 }
 
-static struct Section *find_section_by_index(long index)
+static struct Section *find_section_by_index(int32_t index)
 {
     struct Section *s;
 
@@ -211,9 +207,11 @@ static void bin_cleanup(int debuginfo)
     struct Section *last_progbits;
     struct bin_label *l;
     struct Reloc *r;
-    unsigned long pend;
+    uint64_t pend;
     int h;
 
+    (void)debuginfo;      /* placate optimizers */
+
 #ifdef DEBUG
     fprintf(stdout,
             "bin_cleanup: Sections were initially referenced in this order:\n");
@@ -480,7 +478,7 @@ static void bin_cleanup(int debuginfo)
                 g->vstart =
                     (s->vstart + s->length + g->valign - 1) & ~(g->valign -
                                                                 1);
-                g->flags |= VSTART_DEFINED;
+               g->flags |= VSTART_DEFINED;
                 h++;
                 /* Start and vstart mean the same thing for nobits sections. */
                 if (g->flags & TYPE_NOBITS)
@@ -517,18 +515,24 @@ static void bin_cleanup(int debuginfo)
         saa_rewind(s->contents);
     /* Apply relocations. */
     for (r = relocs; r; r = r->next) {
-        unsigned char *p, *q, mydata[4];
-        long l;
+        uint8_t *p, *q, mydata[8];
+        int64_t l;
 
         saa_fread(r->target->contents, r->posn, mydata, r->bytes);
         p = q = mydata;
         l = *p++;
 
         if (r->bytes > 1) {
-            l += ((long)*p++) << 8;
-            if (r->bytes == 4) {
-                l += ((long)*p++) << 16;
-                l += ((long)*p++) << 24;
+            l += ((int64_t)*p++) << 8;
+            if (r->bytes >= 4) {
+                l += ((int64_t)*p++) << 16;
+                l += ((int64_t)*p++) << 24;
+            }
+            if (r->bytes == 8) {
+                l += ((int64_t)*p++) << 32;
+                l += ((int64_t)*p++) << 40;
+                l += ((int64_t)*p++) << 48;
+                l += ((int64_t)*p++) << 56;
             }
         }
 
@@ -547,12 +551,12 @@ static void bin_cleanup(int debuginfo)
                 l -= s->vstart;
         }
 
-        if (r->bytes == 4)
-            WRITELONG(q, l);
+        if (r->bytes >= 4)
+            WRITEDLONG(q, l);
         else if (r->bytes == 2)
             WRITESHORT(q, l);
         else
-            *q++ = (unsigned char)(l & 0xFF);
+            *q++ = (uint8_t)(l & 0xFF);
         saa_fwrite(r->target->contents, r->posn, mydata, r->bytes);
     }
 
@@ -589,17 +593,17 @@ static void bin_cleanup(int debuginfo)
             fprintf(rf, "-- Program origin ");
             for (h = 61; h; h--)
                 fputc('-', rf);
-            fprintf(rf, "\n\n%08lX\n\n", origin);
+            fprintf(rf, "\n\n%08"PRIX64"\n\n", origin);
         }
         /* Display sections summary. */
         if (map_control & MAP_SUMMARY) {
             fprintf(rf, "-- Sections (summary) ");
             for (h = 57; h; h--)
                 fputc('-', rf);
-            fprintf(rf, "\n\nVstart    Start     Stop      "
+            fprintf(rf, "\n\nVstart            Start             Stop              "
                     "Length    Class     Name\n");
             for (s = sections; s; s = s->next) {
-                fprintf(rf, "%08lX  %08lX  %08lX  %08lX  ",
+                fprintf(rf, "%16"PRIX64"  %16"PRIX64"  %16"PRIX64"  %08"PRIX64"  ",
                         s->vstart, s->start, s->start + s->length,
                         s->length);
                 if (s->flags & TYPE_PROGBITS)
@@ -625,10 +629,10 @@ static void bin_cleanup(int debuginfo)
                     fprintf(rf, "progbits");
                 else
                     fprintf(rf, "nobits");
-                fprintf(rf, "\nlength:    %08lX\nstart:     %08lX"
+                fprintf(rf, "\nlength:    %16"PRIX64"\nstart:     %16"PRIX64""
                         "\nalign:     ", s->length, s->start);
                 if (s->flags & ALIGN_DEFINED)
-                    fprintf(rf, "%08lX", s->align);
+                    fprintf(rf, "%16"PRIX64"", s->align);
                 else
                     fprintf(rf, not_defined);
                 fprintf(rf, "\nfollows:   ");
@@ -636,9 +640,9 @@ static void bin_cleanup(int debuginfo)
                     fprintf(rf, "%s", s->follows);
                 else
                     fprintf(rf, not_defined);
-                fprintf(rf, "\nvstart:    %08lX\nvalign:    ", s->vstart);
+                fprintf(rf, "\nvstart:    %16"PRIX64"\nvalign:    ", s->vstart);
                 if (s->flags & VALIGN_DEFINED)
-                    fprintf(rf, "%08lX", s->valign);
+                    fprintf(rf, "%16"PRIX64"", s->valign);
                 else
                     fprintf(rf, not_defined);
                 fprintf(rf, "\nvfollows:  ");
@@ -651,7 +655,8 @@ static void bin_cleanup(int debuginfo)
         }
         /* Display symbols information. */
         if (map_control & MAP_SYMBOLS) {
-            long segment, offset;
+            int32_t segment;
+            int64_t offset;
 
             fprintf(rf, "-- Symbols ");
             for (h = 68; h; h--)
@@ -664,7 +669,7 @@ static void bin_cleanup(int debuginfo)
                 fprintf(rf, "\n\nValue     Name\n");
                 for (l = no_seg_labels; l; l = l->next) {
                     lookup_label(l->name, &segment, &offset);
-                    fprintf(rf, "%08lX  %s\n", offset, l->name);
+                    fprintf(rf, "%08"PRIX64"  %s\n", offset, l->name);
                 }
                 fprintf(rf, "\n\n");
             }
@@ -673,10 +678,10 @@ static void bin_cleanup(int debuginfo)
                     fprintf(rf, "---- Section %s ", s->name);
                     for (h = 65 - strlen(s->name); h; h--)
                         fputc('-', rf);
-                    fprintf(rf, "\n\nReal      Virtual   Name\n");
+                    fprintf(rf, "\n\nReal              Virtual           Name\n");
                     for (l = s->labels; l; l = l->next) {
                         lookup_label(l->name, &segment, &offset);
-                        fprintf(rf, "%08lX  %08lX  %s\n",
+                        fprintf(rf, "%16"PRIX64"  %16"PRIX64"  %s\n",
                                 s->start + offset, s->vstart + offset,
                                 l->name);
                     }
@@ -725,12 +730,12 @@ static void bin_cleanup(int debuginfo)
     }
 }
 
-static void bin_out(long segto, const void *data, unsigned long type,
-                    long segment, long wrt)
+static void bin_out(int32_t segto, const void *data,
+		    enum out_type type, uint64_t size,
+                    int32_t segment, int32_t wrt)
 {
-    unsigned char *p, mydata[4];
+    uint8_t *p, mydata[8];
     struct Section *s;
-    long realbytes;
 
     if (wrt != NO_SEG) {
         wrt = NO_SEG;           /* continue to do _something_ */
@@ -739,7 +744,7 @@ static void bin_out(long segto, const void *data, unsigned long type,
 
     /* Handle absolute-assembly (structure definitions). */
     if (segto == NO_SEG) {
-        if ((type & OUT_TYPMASK) != OUT_RESERVE)
+        if (type != OUT_RESERVE)
             error(ERR_NONFATAL, "attempt to assemble code in"
                   " [ABSOLUTE] space");
         return;
@@ -752,17 +757,17 @@ static void bin_out(long segto, const void *data, unsigned long type,
 
     /* "Smart" section-type adaptation code. */
     if (!(s->flags & TYPE_DEFINED)) {
-        if ((type & OUT_TYPMASK) == OUT_RESERVE)
+        if (type == OUT_RESERVE)
             s->flags |= TYPE_DEFINED | TYPE_NOBITS;
         else
             s->flags |= TYPE_DEFINED | TYPE_PROGBITS;
     }
 
-    if ((s->flags & TYPE_NOBITS) && ((type & OUT_TYPMASK) != OUT_RESERVE))
-        error(ERR_WARNING, "attempt to initialise memory in a"
+    if ((s->flags & TYPE_NOBITS) && (type != OUT_RESERVE))
+        error(ERR_WARNING, "attempt to initialize memory in a"
               " nobits section: ignored");
 
-    if ((type & OUT_TYPMASK) == OUT_ADDRESS) {
+    if (type == OUT_ADDRESS) {
         if (segment != NO_SEG && !find_section_by_index(segment)) {
             if (segment % 2)
                 error(ERR_NONFATAL, "binary output format does not support"
@@ -774,31 +779,40 @@ static void bin_out(long segto, const void *data, unsigned long type,
         }
         if (s->flags & TYPE_PROGBITS) {
             if (segment != NO_SEG)
-                add_reloc(s, type & OUT_SIZMASK, segment, -1L);
+                add_reloc(s, size, segment, -1L);
             p = mydata;
-            if ((type & OUT_SIZMASK) == 4)
-                WRITELONG(p, *(long *)data);
-            else
-                WRITESHORT(p, *(long *)data);
-            saa_wbytes(s->contents, mydata, type & OUT_SIZMASK);
+	    WRITEADDR(p, *(int64_t *)data, size);
+            saa_wbytes(s->contents, mydata, size);
         }
-        s->length += type & OUT_SIZMASK;
-    } else if ((type & OUT_TYPMASK) == OUT_RAWDATA) {
-        type &= OUT_SIZMASK;
+        s->length += size;
+    } else if (type == OUT_RAWDATA) {
         if (s->flags & TYPE_PROGBITS)
-            saa_wbytes(s->contents, data, type);
-        s->length += type;
-    } else if ((type & OUT_TYPMASK) == OUT_RESERVE) {
-        type &= OUT_SIZMASK;
+            saa_wbytes(s->contents, data, size);
+        s->length += size;
+    } else if (type == OUT_RESERVE) {
         if (s->flags & TYPE_PROGBITS) {
-            error(ERR_WARNING, "uninitialised space declared in"
+            error(ERR_WARNING, "uninitialized space declared in"
                   " %s section: zeroing", s->name);
-            saa_wbytes(s->contents, NULL, type);
+            saa_wbytes(s->contents, NULL, size);
         }
-        s->length += type;
-    } else if ((type & OUT_TYPMASK) == OUT_REL2ADR ||
-               (type & OUT_TYPMASK) == OUT_REL4ADR) {
-        realbytes = ((type & OUT_TYPMASK) == OUT_REL4ADR ? 4 : 2);
+        s->length += size;
+    } else if (type == OUT_REL2ADR || type == OUT_REL4ADR ||
+	       type == OUT_REL8ADR) {
+	int64_t addr = *(int64_t *)data - size;
+	switch (type) {
+	case OUT_REL2ADR:
+	    size = 2;
+	    break;
+	case OUT_REL4ADR:
+	    size = 4;
+	    break;
+	case OUT_REL8ADR:
+	    size = 8;
+	    break;
+	default:
+	    size = 0;		/* Shut up warning */
+	    break;
+	}
         if (segment != NO_SEG && !find_section_by_index(segment)) {
             if (segment % 2)
                 error(ERR_NONFATAL, "binary output format does not support"
@@ -809,19 +823,16 @@ static void bin_out(long segto, const void *data, unsigned long type,
             segment = NO_SEG;
         }
         if (s->flags & TYPE_PROGBITS) {
-            add_reloc(s, realbytes, segment, segto);
+            add_reloc(s, size, segment, segto);
             p = mydata;
-            if (realbytes == 4)
-                WRITELONG(p, *(long *)data - realbytes - s->length);
-            else
-                WRITESHORT(p, *(long *)data - realbytes - s->length);
-            saa_wbytes(s->contents, mydata, realbytes);
+	    WRITEADDR(p, addr - s->length, size);
+            saa_wbytes(s->contents, mydata, size);
         }
-        s->length += realbytes;
+        s->length += size;
     }
 }
 
-static void bin_deflabel(char *name, long segment, long offset,
+static void bin_deflabel(char *name, int32_t segment, int64_t offset,
                          int is_global, char *special)
 {
     (void)segment;              /* Don't warn that this parameter is unused */
@@ -863,7 +874,7 @@ enum { ATTRIB_START, ATTRIB_ALIGN, ATTRIB_FOLLOWS,
 };
 
 static int bin_read_attribute(char **line, int *attribute,
-                              unsigned long *value)
+                              uint64_t *value)
 {
     expr *e;
     int attrib_name_size;
@@ -871,7 +882,7 @@ static int bin_read_attribute(char **line, int *attribute,
     char *exp;
 
     /* Skip whitespace. */
-    while (**line && isspace(**line))
+    while (**line && nasm_isspace(**line))
         (*line)++;
     if (!**line)
         return 0;
@@ -899,12 +910,12 @@ static int bin_read_attribute(char **line, int *attribute,
             *line += 9;
             return 1;
         } else if (!nasm_strnicmp(*line, "nobits", 6) &&
-                   (isspace((*line)[6]) || ((*line)[6] == '\0'))) {
+                   (nasm_isspace((*line)[6]) || ((*line)[6] == '\0'))) {
             *attribute = ATTRIB_NOBITS;
             *line += 6;
             return 1;
         } else if (!nasm_strnicmp(*line, "progbits", 8) &&
-                   (isspace((*line)[8]) || ((*line)[8] == '\0'))) {
+                   (nasm_isspace((*line)[8]) || ((*line)[8] == '\0'))) {
             *attribute = ATTRIB_PROGBITS;
             *line += 8;
             return 1;
@@ -917,7 +928,7 @@ static int bin_read_attribute(char **line, int *attribute,
     if ((*line)[attrib_name_size] != '(') {
         /* Single term (no parenthesis). */
         exp = *line += attrib_name_size;
-        while (**line && !isspace(**line))
+        while (**line && !nasm_isspace(**line))
             (*line)++;
         if (**line) {
             **line = '\0';
@@ -986,14 +997,14 @@ static int bin_read_attribute(char **line, int *attribute,
               " specified in `section' directive.");
         return -1;
     }
-    *value = (unsigned long)reloc_value(e);
+    *value = (uint64_t)reloc_value(e);
     return 1;
 }
 
 static void bin_assign_attributes(struct Section *sec, char *astring)
 {
     int attribute, check;
-    unsigned long value;
+    uint64_t value;
     char *p;
 
     while (1) {                 /* Get the next attribute. */
@@ -1007,7 +1018,7 @@ static void bin_assign_attributes(struct Section *sec, char *astring)
                 break;          /* End of line. */
             else {
                 p = astring;
-                while (*astring && !isspace(*astring))
+                while (*astring && !nasm_isspace(*astring))
                     astring++;
                 if (*astring) {
                     *astring = '\0';
@@ -1096,9 +1107,6 @@ static void bin_assign_attributes(struct Section *sec, char *astring)
             if (sec->flags & FOLLOWS_DEFINED)
                 error(ERR_NONFATAL, "cannot combine `start' and `follows'"
                       " section attributes");
-            else if (value < 0)
-                error(ERR_NONFATAL, "attempt to specify a negative"
-                      " section start address");
             else if ((sec->flags & START_DEFINED) && (value != sec->start))
                 error(ERR_NONFATAL, "section start address redefined");
             else {
@@ -1178,7 +1186,7 @@ static void bin_assign_attributes(struct Section *sec, char *astring)
     }
 }
 
-static void bin_define_section_labels()
+static void bin_define_section_labels(void)
 {
     static int labels_defined = 0;
     struct Section *sec;
@@ -1208,7 +1216,7 @@ static void bin_define_section_labels()
     labels_defined = 1;
 }
 
-static long bin_secname(char *name, int pass, int *bits)
+static int32_t bin_secname(char *name, int pass, int *bits)
 {
     char *p;
     struct Section *sec;
@@ -1238,7 +1246,7 @@ static long bin_secname(char *name, int pass, int *bits)
     /* Attempt to find the requested section.  If it does not
      * exist, create it. */
     p = name;
-    while (*p && !isspace(*p))
+    while (*p && !nasm_isspace(*p))
         p++;
     if (*p)
         *p++ = '\0';
@@ -1279,7 +1287,7 @@ static int bin_directive(char *directive, char *args, int pass)
     /* Handle ORG directive */
     if (!nasm_stricmp(directive, "org")) {
         struct tokenval tokval;
-        unsigned long value;
+        uint64_t value;
         expr *e;
 
         stdscan_reset();
@@ -1363,13 +1371,15 @@ static void bin_filename(char *inname, char *outname, efunc error)
     outfile = outname;
 }
 
-static long bin_segbase(long segment)
+static int32_t bin_segbase(int32_t segment)
 {
     return segment;
 }
 
 static int bin_set_info(enum geninfo type, char **val)
 {
+    (void)type;
+    (void)val;
     return 0;
 }
 
@@ -1381,6 +1391,7 @@ static void bin_init(FILE * afp, efunc errfunc, ldfunc ldef, evalfunc eval)
     (void)eval;                 /* Don't warn that this parameter is unused. */
     (void)ldef;                 /* Placate optimizers. */
 
+    maxbits = 64;               /* Support 64-bit Segments */
     relocs = NULL;
     reloctail = &relocs;
     origin_defined = 0;
@@ -1423,7 +1434,7 @@ struct ofmt of_bin = {
 };
 
 /* This is needed for bin_define_section_labels() */
-struct ofmt *bin_get_ofmt()
+struct ofmt *bin_get_ofmt(void)
 {
     return &of_bin;
 }
