@@ -3,21 +3,26 @@
  *
  * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
  * Julian Hall. All rights reserved. The software is
- * redistributable under the licence given in the file "Licence"
+ * redistributable under the license given in the file "LICENSE"
  * distributed in the NASM archive.
  */
 
+#include "compiler.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <time.h>
+#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "saa.h"
+#include "raa.h"
 #include "outform.h"
 
-#if defined(OF_COFF) || defined(OF_WIN32)
+#if defined(OF_COFF) || defined(OF_WIN32) || defined(OF_WIN64)
 
 /*
  * Notes on COFF:
@@ -52,6 +57,9 @@
  * seem to fit at all. In fact, Win32 even includes non-linked
  * sections such as .drectve in this calculation.
  *
+ * Newer versions of MASM seem to have changed this to be zero, and
+ * that apparently matches the COFF spec, so go with that.
+ *
  * (5) Standard COFF does something very strange to common
  * variables: the relocation point for a common variable is as far
  * _before_ the variable as its size stretches out _after_ it. So
@@ -60,27 +68,41 @@
  */
 
 /* Flag which version of COFF we are currently outputting. */
-static int win32;
+static bool win32, win64;
+
+static int32_t imagebase_sect;
+#define WRT_IMAGEBASE "..imagebase"
 
 struct Reloc {
     struct Reloc *next;
-    long address;               /* relative to _start_ of section */
-    long symbol;                /* symbol number */
+    int32_t address;		/* relative to _start_ of section */
+    int32_t symbol;		/* symbol number */
     enum {
         SECT_SYMBOLS,
         ABS_SYMBOL,
         REAL_SYMBOLS
     } symbase;                  /* relocation for symbol number :) */
-    int relative;               /* TRUE or FALSE */
+    int16_t type;
 };
 
+/* possible values for Reloc->type */
+#define IMAGE_REL_AMD64_ADDR64		0x0001
+#define IMAGE_REL_AMD64_ADDR32		0x0002
+#define IMAGE_REL_AMD64_ADDR32NB	0x0003
+#define IMAGE_REL_AMD64_REL32		0x0004
+#define IMAGE_REL_I386_DIR32		0x0006
+#define IMAGE_REL_I386_DIR32NB		0x0007
+#define IMAGE_REL_I386_REL32		0x0014
+
 struct Symbol {
     char name[9];
-    long strpos;                /* string table position of name */
+    int32_t strpos;		/* string table position of name */
+    int32_t value;		/* address, or COMMON variable size */
     int section;                /* section number where it's defined
                                  * - in COFF codes, not NASM codes */
-    int is_global;              /* is it a global symbol or not? */
-    long value;                 /* address, or COMMON variable size */
+    bool is_global;              /* is it a global symbol or not? */
+    int16_t type;		/* 0 - notype, 0x20 - function */
+    int32_t namlen;		/* full name length */
 };
 
 static FILE *coffp;
@@ -89,58 +111,74 @@ static char coff_infile[FILENAME_MAX];
 
 struct Section {
     struct SAA *data;
-    unsigned long len;
+    uint32_t len;
     int nrelocs;
-    long index;
+    int32_t index;
     struct Reloc *head, **tail;
-    unsigned long flags;        /* section flags */
+    uint32_t flags;        /* section flags */
     char name[9];
-    long pos, relpos;
+    int32_t pos, relpos;
 };
 
-#define TEXT_FLAGS (win32 ? 0x60500020L : 0x20L)
-#define DATA_FLAGS (win32 ? 0xC0300040L : 0x40L)
-#define BSS_FLAGS (win32 ? 0xC0300080L : 0x80L)
+#define TEXT_FLAGS ((win32 | win64) ? 0x60500020L : 0x20L)
+#define DATA_FLAGS ((win32 | win64) ? 0xC0300040L : 0x40L)
+#define BSS_FLAGS ((win32 | win64) ? 0xC0300080L : 0x80L)
 #define INFO_FLAGS 0x00100A00L
-#define RDATA_FLAGS (win32 ? 0x40400040L : 0x40L)
+#define RDATA_FLAGS ((win32 | win64) ? 0x40400040L : 0x40L)
 
 #define SECT_DELTA 32
 static struct Section **sects;
 static int nsects, sectlen;
 
 static struct SAA *syms;
-static unsigned long nsyms;
+static uint32_t nsyms;
 
-static long def_seg;
+static int32_t def_seg;
 
 static int initsym;
 
 static struct RAA *bsym, *symval;
 
 static struct SAA *strs;
-static unsigned long strslen;
+static uint32_t strslen;
 
 static void coff_gen_init(FILE *, efunc);
-static void coff_sect_write(struct Section *, const unsigned char *,
-                            unsigned long);
+static void coff_sect_write(struct Section *, const uint8_t *,
+                            uint32_t);
 static void coff_write(void);
-static void coff_section_header(char *, long, long, long, long, int, long);
+static void coff_section_header(char *, int32_t, int32_t, int32_t, int32_t, int, int32_t);
 static void coff_write_relocs(struct Section *);
 static void coff_write_symbols(void);
 
 static void coff_win32_init(FILE * fp, efunc errfunc,
                             ldfunc ldef, evalfunc eval)
 {
-    win32 = TRUE;
-    (void)ldef;                 /* placate optimisers */
+    win32 = true; win64 = false;
+    (void)ldef;                 /* placate optimizers */
+    (void)eval;
     coff_gen_init(fp, errfunc);
 }
 
+static void coff_win64_init(FILE * fp, efunc errfunc,
+                            ldfunc ldef, evalfunc eval)
+{
+    extern struct ofmt of_win64;
+
+    maxbits = 64;
+    win32 = false; win64 = true;
+    (void)ldef;                 /* placate optimizers */
+    (void)eval;
+    coff_gen_init(fp, errfunc);
+    imagebase_sect = seg_alloc()+1;
+    ldef(WRT_IMAGEBASE,imagebase_sect,0,NULL,false,false,&of_win64,errfunc);
+}
+
 static void coff_std_init(FILE * fp, efunc errfunc, ldfunc ldef,
                           evalfunc eval)
 {
-    win32 = FALSE;
-    (void)ldef;                 /* placate optimisers */
+    win32 = win64 = false;
+    (void)ldef;                 /* placate optimizers */
+    (void)eval;
     coff_gen_init(fp, errfunc);
 }
 
@@ -151,11 +189,11 @@ static void coff_gen_init(FILE * fp, efunc errfunc)
     error = errfunc;
     sects = NULL;
     nsects = sectlen = 0;
-    syms = saa_init((long)sizeof(struct Symbol));
+    syms = saa_init(sizeof(struct Symbol));
     nsyms = 0;
     bsym = raa_init();
     symval = raa_init();
-    strs = saa_init(1L);
+    strs = saa_init(1);
     strslen = 0;
     def_seg = seg_alloc();
 }
@@ -186,14 +224,14 @@ static void coff_cleanup(int debuginfo)
     saa_free(strs);
 }
 
-static int coff_make_section(char *name, unsigned long flags)
+static int coff_make_section(char *name, uint32_t flags)
 {
     struct Section *s;
 
     s = nasm_malloc(sizeof(*s));
 
     if (flags != BSS_FLAGS)
-        s->data = saa_init(1L);
+        s->data = saa_init(1);
     else
         s->data = NULL;
     s->head = NULL;
@@ -208,31 +246,36 @@ static int coff_make_section(char *name, unsigned long flags)
     s->name[8] = '\0';
     s->flags = flags;
 
-    if (nsects >= sectlen)
-        sects =
-            nasm_realloc(sects, (sectlen += SECT_DELTA) * sizeof(*sects));
+    if (nsects >= sectlen) {
+	sectlen += SECT_DELTA;
+        sects = nasm_realloc(sects, sectlen * sizeof(*sects));
+    }
     sects[nsects++] = s;
 
     return nsects - 1;
 }
 
-static long coff_section_names(char *name, int pass, int *bits)
+static int32_t coff_section_names(char *name, int pass, int *bits)
 {
     char *p;
-    unsigned long flags, align_and = ~0L, align_or = 0L;
+    uint32_t flags, align_and = ~0L, align_or = 0L;
     int i;
 
     /*
-     * Default is 32 bits.
+     * Set default bits.
      */
-    if (!name)
-        *bits = 32;
+    if (!name) {
+        if(win64)
+            *bits = 64;
+        else
+            *bits = 32;
+    }
 
     if (!name)
         return def_seg;
 
     p = name;
-    while (*p && !isspace(*p))
+    while (*p && !nasm_isspace(*p))
         p++;
     if (*p)
         *p++ = '\0';
@@ -243,15 +286,15 @@ static long coff_section_names(char *name, int pass, int *bits)
     }
     flags = 0;
 
-    while (*p && isspace(*p))
+    while (*p && nasm_isspace(*p))
         p++;
     while (*p) {
         char *q = p;
-        while (*p && !isspace(*p))
+        while (*p && !nasm_isspace(*p))
             p++;
         if (*p)
             *p++ = '\0';
-        while (*p && isspace(*p))
+        while (*p && nasm_isspace(*p))
             p++;
 
         if (!nasm_stricmp(q, "code") || !nasm_stricmp(q, "text")) {
@@ -259,7 +302,7 @@ static long coff_section_names(char *name, int pass, int *bits)
         } else if (!nasm_stricmp(q, "data")) {
             flags = DATA_FLAGS;
         } else if (!nasm_stricmp(q, "rdata")) {
-            if (win32)
+            if (win32 | win64)
                 flags = RDATA_FLAGS;
             else {
                 flags = DATA_FLAGS;     /* gotta do something */
@@ -269,7 +312,7 @@ static long coff_section_names(char *name, int pass, int *bits)
         } else if (!nasm_stricmp(q, "bss")) {
             flags = BSS_FLAGS;
         } else if (!nasm_stricmp(q, "info")) {
-            if (win32)
+            if (win32 | win64)
                 flags = INFO_FLAGS;
             else {
                 flags = DATA_FLAGS;     /* gotta do something */
@@ -277,7 +320,7 @@ static long coff_section_names(char *name, int pass, int *bits)
                       " informational sections");
             }
         } else if (!nasm_strnicmp(q, "align=", 6)) {
-            if (!win32)
+            if (!(win32 | win64))
                 error(ERR_NONFATAL, "standard COFF does not support"
                       " section alignment specification");
             else {
@@ -318,6 +361,10 @@ static long coff_section_names(char *name, int pass, int *bits)
                 flags = RDATA_FLAGS;
             else if (!strcmp(name, ".bss"))
                 flags = BSS_FLAGS;
+	    else if (win64 && !strcmp(name, ".pdata"))
+		flags = 0x40300040; /* rdata align=4 */
+	    else if (win64 && !strcmp(name, ".xdata"))
+		flags = 0x40400040; /* rdate align=8 */
             else
                 flags = TEXT_FLAGS;
         }
@@ -335,7 +382,7 @@ static long coff_section_names(char *name, int pass, int *bits)
     return sects[i]->index;
 }
 
-static void coff_deflabel(char *name, long segment, long offset,
+static void coff_deflabel(char *name, int32_t segment, int64_t offset,
                           int is_global, char *special)
 {
     int pos = strslen + 4;
@@ -346,22 +393,26 @@ static void coff_deflabel(char *name, long segment, long offset,
               " special symbol types");
 
     if (name[0] == '.' && name[1] == '.' && name[2] != '@') {
-        error(ERR_NONFATAL, "unrecognised special symbol `%s'", name);
+	if (strcmp(name,WRT_IMAGEBASE))
+            error(ERR_NONFATAL, "unrecognized special symbol `%s'", name);
         return;
     }
 
     if (strlen(name) > 8) {
-        saa_wbytes(strs, name, (long)(1 + strlen(name)));
-        strslen += 1 + strlen(name);
+	size_t nlen = strlen(name)+1;
+        saa_wbytes(strs, name, nlen);
+	strslen += nlen;
     } else
         pos = -1;
 
     sym = saa_wstruct(syms);
 
     sym->strpos = pos;
+    sym->namlen = strlen(name);
     if (pos == -1)
         strcpy(sym->name, name);
     sym->is_global = !!is_global;
+    sym->type = 0;		/* Default to T_NULL (no type) */
     if (segment == NO_SEG)
         sym->section = -1;      /* absolute symbol */
     else {
@@ -373,7 +424,7 @@ static void coff_deflabel(char *name, long segment, long offset,
                 break;
             }
         if (!sym->section)
-            sym->is_global = TRUE;
+            sym->is_global = true;
     }
     if (is_global == 2)
         sym->value = offset;
@@ -384,8 +435,9 @@ static void coff_deflabel(char *name, long segment, long offset,
      * define the references from external-symbol segment numbers
      * to these symbol records.
      */
-    if (sym->section == 0)
+    if (sym->section == 0) {
         bsym = raa_write(bsym, segment, nsyms);
+    }
 
     if (segment != NO_SEG)
         symval = raa_write(symval, segment, sym->section ? 0 : sym->value);
@@ -393,8 +445,8 @@ static void coff_deflabel(char *name, long segment, long offset,
     nsyms++;
 }
 
-static long coff_add_reloc(struct Section *sect, long segment,
-                           int relative)
+static int32_t coff_add_reloc(struct Section *sect, int32_t segment,
+				int16_t type)
 {
     struct Reloc *r;
 
@@ -417,34 +469,32 @@ static long coff_add_reloc(struct Section *sect, long segment,
         if (r->symbase == REAL_SYMBOLS)
             r->symbol = raa_read(bsym, segment);
     }
-    r->relative = relative;
+    r->type = type;
 
     sect->nrelocs++;
 
     /*
      * Return the fixup for standard COFF common variables.
      */
-    if (r->symbase == REAL_SYMBOLS && !win32)
+    if (r->symbase == REAL_SYMBOLS && !(win32 | win64))
         return raa_read(symval, segment);
     else
         return 0;
 }
 
-static void coff_out(long segto, const void *data, unsigned long type,
-                     long segment, long wrt)
+static void coff_out(int32_t segto, const void *data,
+		     enum out_type type, uint64_t size,
+                     int32_t segment, int32_t wrt)
 {
     struct Section *s;
-    long realbytes = type & OUT_SIZMASK;
-    unsigned char mydata[4], *p;
+    uint8_t mydata[8], *p;
     int i;
 
-    if (wrt != NO_SEG) {
+    if (wrt != NO_SEG && !win64) {
         wrt = NO_SEG;           /* continue to do _something_ */
         error(ERR_NONFATAL, "WRT not supported by COFF output formats");
     }
 
-    type &= OUT_TYPMASK;
-
     /*
      * handle absolute-assembly (structure definitions)
      */
@@ -469,14 +519,19 @@ static void coff_out(long segto, const void *data, unsigned long type,
             s = sects[nsects - 1];
     }
 
+    /* magically default to 'wrt ..imagebase' in .pdata and .xdata */
+    if (win64 && wrt == NO_SEG &&
+	(!strcmp(s->name,".pdata") || !strcmp(s->name,".xdata")))
+	wrt = imagebase_sect;
+
     if (!s->data && type != OUT_RESERVE) {
-        error(ERR_WARNING, "attempt to initialise memory in"
+        error(ERR_WARNING, "attempt to initialize memory in"
               " BSS section `%s': ignored", s->name);
         if (type == OUT_REL2ADR)
-            realbytes = 2;
+            size = 2;
         else if (type == OUT_REL4ADR)
-            realbytes = 4;
-        s->len += realbytes;
+            size = 4;
+        s->len += size;
         return;
     }
 
@@ -484,62 +539,84 @@ static void coff_out(long segto, const void *data, unsigned long type,
         if (s->data) {
             error(ERR_WARNING, "uninitialised space declared in"
                   " non-BSS section `%s': zeroing", s->name);
-            coff_sect_write(s, NULL, realbytes);
+            coff_sect_write(s, NULL, size);
         } else
-            s->len += realbytes;
+            s->len += size;
     } else if (type == OUT_RAWDATA) {
         if (segment != NO_SEG)
             error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
-        coff_sect_write(s, data, realbytes);
+        coff_sect_write(s, data, size);
     } else if (type == OUT_ADDRESS) {
-        if (realbytes != 4 && (segment != NO_SEG || wrt != NO_SEG))
-            error(ERR_NONFATAL, "COFF format does not support non-32-bit"
-                  " relocations");
-        else {
-            long fix = 0;
-            if (segment != NO_SEG || wrt != NO_SEG) {
-                if (wrt != NO_SEG) {
-                    error(ERR_NONFATAL, "COFF format does not support"
-                          " WRT types");
-                } else if (segment % 2) {
-                    error(ERR_NONFATAL, "COFF format does not support"
-                          " segment base references");
-                } else
-                    fix = coff_add_reloc(s, segment, FALSE);
+        if (!(win64)) {
+            if (size != 4 && (segment != NO_SEG || wrt != NO_SEG))
+                error(ERR_NONFATAL, "COFF format does not support non-32-bit"
+                      " relocations");
+            else {
+                int32_t fix = 0;
+                if (segment != NO_SEG || wrt != NO_SEG) {
+                    if (wrt != NO_SEG) {
+                        error(ERR_NONFATAL, "COFF format does not support"
+                              " WRT types");
+                    } else if (segment % 2) {
+                        error(ERR_NONFATAL, "COFF format does not support"
+                              " segment base references");
+                    } else
+                        fix = coff_add_reloc(s, segment, IMAGE_REL_I386_DIR32);
+                }
+                p = mydata;
+                WRITELONG(p, *(int64_t *)data + fix);
+                coff_sect_write(s, mydata, size);
             }
+        } else {
+            int32_t fix = 0;
             p = mydata;
-            WRITELONG(p, *(long *)data + fix);
-            coff_sect_write(s, mydata, realbytes);
+            if (size == 8) {
+	        if (wrt == imagebase_sect) {
+		    error(ERR_NONFATAL, "operand size mismatch: 'wrt "
+			WRT_IMAGEBASE "' is a 32-bit operand");
+		}
+                fix = coff_add_reloc(s, segment, IMAGE_REL_AMD64_ADDR64);
+                WRITEDLONG(p, *(int64_t *)data + fix);
+                coff_sect_write(s, mydata, size);
+            } else {
+                fix = coff_add_reloc(s, segment,
+			wrt == imagebase_sect ?	IMAGE_REL_AMD64_ADDR32NB:
+						IMAGE_REL_AMD64_ADDR32);
+                WRITELONG(p, *(int64_t *)data + fix);
+                coff_sect_write(s, mydata, size);
+            }
         }
     } else if (type == OUT_REL2ADR) {
         error(ERR_NONFATAL, "COFF format does not support 16-bit"
               " relocations");
     } else if (type == OUT_REL4ADR) {
-        if (segment == segto)
+        if (segment == segto && !(win64))  /* Acceptable for RIP-relative */
             error(ERR_PANIC, "intra-segment OUT_REL4ADR");
         else if (segment == NO_SEG && win32)
             error(ERR_NONFATAL, "Win32 COFF does not correctly support"
                   " relative references to absolute addresses");
         else {
-            long fix = 0;
+            int32_t fix = 0;
             if (segment != NO_SEG && segment % 2) {
                 error(ERR_NONFATAL, "COFF format does not support"
                       " segment base references");
             } else
-                fix = coff_add_reloc(s, segment, TRUE);
+                fix = coff_add_reloc(s, segment,
+			win64 ? IMAGE_REL_AMD64_REL32 : IMAGE_REL_I386_REL32);
             p = mydata;
-            if (win32) {
-                WRITELONG(p, *(long *)data + 4 - realbytes + fix);
+            if (win32 | win64) {
+                WRITELONG(p, *(int64_t *)data + 4 - size + fix);
             } else {
-                WRITELONG(p, *(long *)data - (realbytes + s->len) + fix);
+                WRITELONG(p, *(int64_t *)data - (size + s->len) + fix);
             }
             coff_sect_write(s, mydata, 4L);
         }
+
     }
 }
 
 static void coff_sect_write(struct Section *sect,
-                            const unsigned char *data, unsigned long len)
+                            const uint8_t *data, uint32_t len)
 {
     saa_wbytes(sect->data, data, len);
     sect->len += len;
@@ -553,7 +630,7 @@ typedef struct tagString {
 
 #define EXPORT_SECTION_NAME ".drectve"
 #define EXPORT_SECTION_FLAGS INFO_FLAGS
-/* 
+/*
 #define EXPORT_SECTION_NAME ".text"
 #define EXPORT_SECTION_FLAGS TEXT_FLAGS
 */
@@ -595,7 +672,7 @@ void AddExport(char *name)
 void BuildExportTable(void)
 {
     STRING *rvp = Exports, *next;
-    unsigned char buf[256];
+    uint8_t buf[256];
     int len;
     if (rvp == NULL)
         return;
@@ -622,11 +699,11 @@ static int coff_directives(char *directive, char *value, int pass)
         if (pass == 2)
             return 1;           /* ignore in pass two */
         name = q = value;
-        while (*q && !isspace(*q))
+        while (*q && !nasm_isspace(*q))
             q++;
-        if (isspace(*q)) {
+        if (nasm_isspace(*q)) {
             *q++ = '\0';
-            while (*q && isspace(*q))
+            while (*q && nasm_isspace(*q))
                 q++;
         }
 
@@ -635,21 +712,84 @@ static int coff_directives(char *directive, char *value, int pass)
             return 1;
         }
         if (*q) {
-            error(ERR_NONFATAL, "unrecognised export qualifier `%s'", q);
+            error(ERR_NONFATAL, "unrecognized export qualifier `%s'", q);
             return 1;
         }
         AddExport(name);
         return 1;
+    } else if (win32 && !strcmp(directive,"safeseh")) {
+	static int sxseg=-1;
+	int i;
+	if (sxseg==-1)
+	{   for (i = 0; i < nsects; i++)
+		if (!strcmp(".sxdata",sects[i]->name))
+		    break;
+	    if (i == nsects)
+		sxseg = coff_make_section(".sxdata",0x200);
+	    else
+		sxseg = i;
+	}
+	if (pass==2) {
+	    uint32_t n;
+	    saa_rewind(syms);
+	    for (n = 0; n < nsyms; n++) {
+		struct Symbol *sym = saa_rstruct(syms);
+		bool equals;
+
+		/* sym->strpos is biased by 4, because symbol
+		 * table is prefixed with table length */
+		if (sym->strpos >=4) {
+		    char *name = nasm_malloc(sym->namlen+1);
+		    saa_fread(strs, sym->strpos-4, name, sym->namlen);
+		    name[sym->namlen] = '\0';
+		    equals = !strcmp(value,name);
+		    nasm_free(name);
+		}
+		else
+		    equals = !strcmp(value,sym->name);
+
+		if (equals) {
+		    /* this value arithmetics effectively reflects
+		     * initsym in coff_write(): 2 for file, 1 for
+		     * .absolute and two per each section */
+		    unsigned char value[4],*p=value;
+		    WRITELONG(p,n + 2 + 1 + nsects*2);
+		    coff_sect_write(sects[sxseg],value,4);
+		    sym->type = 0x20;
+		    break;
+		}
+	    }
+	    if (n == nsyms) {
+		error(ERR_NONFATAL,
+		      "`safeseh' directive requires valid symbol");
+	    }
+	}
+	return 1;
     }
     return 0;
 }
 
 static void coff_write(void)
 {
-    long pos, sympos, vsize;
+    int32_t pos, sympos, vsize;
     int i;
 
     BuildExportTable();         /* fill in the .drectve section with -export's */
+
+    if (win32) {
+	/* add default value for @feat.00, this allows to 'link /safeseh' */
+	uint32_t n;
+
+	saa_rewind(syms);
+	for (n = 0; n < nsyms; n++) {
+            struct Symbol *sym = saa_rstruct(syms);
+	    if (sym->strpos == -1 && !strcmp("@feat.00",sym->name))
+		break;
+	}
+	if (n == nsyms)
+	    coff_deflabel("@feat.00",NO_SEG,1,0,NULL);
+    }
+
     /*
      * Work out how big the file will get. Calculate the start of
      * the `real' symbols at the same time.
@@ -671,14 +811,17 @@ static void coff_write(void)
     /*
      * Output the COFF header.
      */
-    fwriteshort(0x14C, coffp);  /* MACHINE_i386 */
-    fwriteshort(nsects, coffp); /* number of sections */
-    fwritelong(time(NULL), coffp);      /* time stamp */
-    fwritelong(sympos, coffp);
-    fwritelong(nsyms + initsym, coffp);
-    fwriteshort(0, coffp);      /* no optional header */
+    if (win64)
+        fwriteint16_t(0x8664, coffp);  /* MACHINE_x86-64 */
+    else
+        fwriteint16_t(0x014C, coffp);  /* MACHINE_i386 */
+    fwriteint16_t(nsects, coffp); /* number of sections */
+    fwriteint32_t(time(NULL), coffp);      /* time stamp */
+    fwriteint32_t(sympos, coffp);
+    fwriteint32_t(nsyms + initsym, coffp);
+    fwriteint16_t(0, coffp);      /* no optional header */
     /* Flags: 32-bit, no line numbers. Win32 doesn't even bother with them. */
-    fwriteshort(win32 ? 0 : 0x104, coffp);
+    fwriteint16_t((win32 | win64) ? 0 : 0x104, coffp);
 
     /*
      * Output the section headers.
@@ -704,28 +847,30 @@ static void coff_write(void)
      * Output the symbol and string tables.
      */
     coff_write_symbols();
-    fwritelong(strslen + 4, coffp);     /* length includes length count */
+    fwriteint32_t(strslen + 4, coffp);     /* length includes length count */
     saa_fpwrite(strs, coffp);
 }
 
-static void coff_section_header(char *name, long vsize,
-                                long datalen, long datapos,
-                                long relpos, int nrelocs, long flags)
+static void coff_section_header(char *name, int32_t vsize,
+                                int32_t datalen, int32_t datapos,
+                                int32_t relpos, int nrelocs, int32_t flags)
 {
     char padname[8];
 
+    (void)vsize;
+
     memset(padname, 0, 8);
     strncpy(padname, name, 8);
     fwrite(padname, 8, 1, coffp);
-    fwritelong(vsize, coffp);
-    fwritelong(0L, coffp);      /* RVA/offset - we ignore */
-    fwritelong(datalen, coffp);
-    fwritelong(datapos, coffp);
-    fwritelong(relpos, coffp);
-    fwritelong(0L, coffp);      /* no line numbers - we don't do 'em */
-    fwriteshort(nrelocs, coffp);
-    fwriteshort(0, coffp);      /* again, no line numbers */
-    fwritelong(flags, coffp);
+    fwriteint32_t(0, coffp);	/* Virtual size field - set to 0 or vsize */
+    fwriteint32_t(0L, coffp);      /* RVA/offset - we ignore */
+    fwriteint32_t(datalen, coffp);
+    fwriteint32_t(datapos, coffp);
+    fwriteint32_t(relpos, coffp);
+    fwriteint32_t(0L, coffp);      /* no line numbers - we don't do 'em */
+    fwriteint16_t(nrelocs, coffp);
+    fwriteint16_t(0, coffp);      /* again, no line numbers */
+    fwriteint32_t(flags, coffp);
 }
 
 static void coff_write_relocs(struct Section *s)
@@ -733,23 +878,17 @@ static void coff_write_relocs(struct Section *s)
     struct Reloc *r;
 
     for (r = s->head; r; r = r->next) {
-        fwritelong(r->address, coffp);
-        fwritelong(r->symbol + (r->symbase == REAL_SYMBOLS ? initsym :
+        fwriteint32_t(r->address, coffp);
+        fwriteint32_t(r->symbol + (r->symbase == REAL_SYMBOLS ? initsym :
                                 r->symbase == ABS_SYMBOL ? initsym - 1 :
                                 r->symbase == SECT_SYMBOLS ? 2 : 0),
                    coffp);
-        /*
-         * Strange: Microsoft's COFF documentation says 0x03 for an
-         * absolute relocation, but both Visual C++ and DJGPP agree
-         * that in fact it's 0x06. I'll use 0x06 until someone
-         * argues.
-         */
-        fwriteshort(r->relative ? 0x14 : 0x06, coffp);
+	fwriteint16_t(r->type, coffp);
     }
 }
 
-static void coff_symbol(char *name, long strpos, long value,
-                        int section, int type, int aux)
+static void coff_symbol(char *name, int32_t strpos, int32_t value,
+                        int section, int type, int storageclass, int aux)
 {
     char padname[8];
 
@@ -758,25 +897,25 @@ static void coff_symbol(char *name, long strpos, long value,
         strncpy(padname, name, 8);
         fwrite(padname, 8, 1, coffp);
     } else {
-        fwritelong(0L, coffp);
-        fwritelong(strpos, coffp);
+        fwriteint32_t(0, coffp);
+        fwriteint32_t(strpos, coffp);
     }
-    fwritelong(value, coffp);
-    fwriteshort(section, coffp);
-    fwriteshort(0, coffp);
-    fputc(type, coffp);
+    fwriteint32_t(value, coffp);
+    fwriteint16_t(section, coffp);
+    fwriteint16_t(type, coffp);
+    fputc(storageclass, coffp);
     fputc(aux, coffp);
 }
 
 static void coff_write_symbols(void)
 {
     char filename[18];
-    unsigned long i;
+    uint32_t i;
 
     /*
      * The `.file' record, and the file name auxiliary record.
      */
-    coff_symbol(".file", 0L, 0L, -2, 0x67, 1);
+    coff_symbol(".file", 0L, 0L, -2, 0, 0x67, 1);
     memset(filename, 0, 18);
     strncpy(filename, coff_infile, 18);
     fwrite(filename, 18, 1, coffp);
@@ -786,17 +925,17 @@ static void coff_write_symbols(void)
      */
     memset(filename, 0, 18);    /* useful zeroed buffer */
 
-    for (i = 0; i < nsects; i++) {
-        coff_symbol(sects[i]->name, 0L, 0L, i + 1, 3, 1);
-        fwritelong(sects[i]->len, coffp);
-        fwriteshort(sects[i]->nrelocs, coffp);
+    for (i = 0; i < (uint32_t) nsects; i++) {
+        coff_symbol(sects[i]->name, 0L, 0L, i + 1, 0, 3, 1);
+        fwriteint32_t(sects[i]->len, coffp);
+        fwriteint16_t(sects[i]->nrelocs, coffp);
         fwrite(filename, 12, 1, coffp);
     }
 
     /*
      * The absolute symbol, for relative-to-absolute relocations.
      */
-    coff_symbol(".absolut", 0L, 0L, -1, 3, 0);
+    coff_symbol(".absolut", 0L, 0L, -1, 0, 3, 0);
 
     /*
      * The real symbols.
@@ -806,11 +945,11 @@ static void coff_write_symbols(void)
         struct Symbol *sym = saa_rstruct(syms);
         coff_symbol(sym->strpos == -1 ? sym->name : NULL,
                     sym->strpos, sym->value, sym->section,
-                    sym->is_global ? 2 : 3, 0);
+                    sym->type, sym->is_global ? 2 : 3, 0);
     }
 }
 
-static long coff_segbase(long segment)
+static int32_t coff_segbase(int32_t segment)
 {
     return segment;
 }
@@ -827,18 +966,12 @@ static void coff_win32_filename(char *inname, char *outname, efunc error)
     standard_extension(inname, outname, ".obj", error);
 }
 
-static const char *coff_stdmac[] = {
-    "%define __SECT__ [section .text]",
-    "%macro __NASM_CDecl__ 1",
-    "%endmacro",
-    "%imacro export 1+.nolist",
-    "[export %1]",
-    "%endmacro",
-    NULL
-};
+extern macros_t coff_stdmac[];
 
 static int coff_set_info(enum geninfo type, char **val)
 {
+    (void)type;
+    (void)val;
     return 0;
 }
 #endif                          /* defined(OF_COFF) || defined(OF_WIN32) */
@@ -886,3 +1019,25 @@ struct ofmt of_win32 = {
 };
 
 #endif
+
+#ifdef OF_WIN64
+
+struct ofmt of_win64 = {
+    "Microsoft Win64 (x86-64) object files",
+    "win64",
+    NULL,
+    null_debug_arr,
+    &null_debug_form,
+    coff_stdmac,
+    coff_win64_init,
+    coff_set_info,
+    coff_out,
+    coff_deflabel,
+    coff_section_names,
+    coff_segbase,
+    coff_directives,
+    coff_win32_filename,
+    coff_cleanup
+};
+
+#endif
