@@ -3,7 +3,7 @@
  *
  * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
  * Julian Hall. All rights reserved. The software is
- * redistributable under the licence given in the file "Licence"
+ * redistributable under the license given in the file "LICENSE"
  * distributed in the NASM archive.
  */
 
@@ -346,8 +346,11 @@ static ObjRecord *obj_x(ObjRecord * orp, uint32_t val)
         orp->x_size = 32;
     if (val > 0xFFFF)
         orp = obj_force(orp, 32);
-    if (orp->x_size == 32)
-        return (obj_dword(orp, val));
+    if (orp->x_size == 32) {
+	ObjRecord *nxt = obj_dword(orp, val);
+	nxt->x_size = 32;	/* x_size is cleared when a record overflows */
+	return nxt;
+    }
     orp->x_size = 16;
     return (obj_word(orp, val));
 }
@@ -1288,19 +1291,19 @@ static int32_t obj_segment(char *name, int pass, int *bits)
         while (*name == '.')
             name++;             /* hack, but a documented one */
         p = name;
-        while (*p && !isspace(*p))
+        while (*p && !nasm_isspace(*p))
             p++;
         if (*p) {
             *p++ = '\0';
-            while (*p && isspace(*p))
+            while (*p && nasm_isspace(*p))
                 *p++ = '\0';
         }
         while (*p) {
-            while (*p && !isspace(*p))
+            while (*p && !nasm_isspace(*p))
                 p++;
             if (*p) {
                 *p++ = '\0';
-                while (*p && isspace(*p))
+                while (*p && nasm_isspace(*p))
                     *p++ = '\0';
             }
 
@@ -1522,11 +1525,11 @@ static int obj_directive(char *directive, char *value, int pass)
             while (*q == '.')
                 q++;            /* hack, but a documented one */
             v = q;
-            while (*q && !isspace(*q))
+            while (*q && !nasm_isspace(*q))
                 q++;
-            if (isspace(*q)) {
+            if (nasm_isspace(*q)) {
                 *q++ = '\0';
-                while (*q && isspace(*q))
+                while (*q && nasm_isspace(*q))
                     q++;
             }
             /*
@@ -1565,11 +1568,11 @@ static int obj_directive(char *directive, char *value, int pass)
 
             while (*q) {
                 p = q;
-                while (*q && !isspace(*q))
+                while (*q && !nasm_isspace(*q))
                     q++;
-                if (isspace(*q)) {
+                if (nasm_isspace(*q)) {
                     *q++ = '\0';
-                    while (*q && isspace(*q))
+                    while (*q && nasm_isspace(*q))
                         q++;
                 }
                 /*
@@ -1630,20 +1633,20 @@ static int obj_directive(char *directive, char *value, int pass)
         if (pass == 2)
             return 1;           /* ignore in pass two */
         extname = q = value;
-        while (*q && !isspace(*q))
+        while (*q && !nasm_isspace(*q))
             q++;
-        if (isspace(*q)) {
+        if (nasm_isspace(*q)) {
             *q++ = '\0';
-            while (*q && isspace(*q))
+            while (*q && nasm_isspace(*q))
                 q++;
         }
 
         libname = q;
-        while (*q && !isspace(*q))
+        while (*q && !nasm_isspace(*q))
             q++;
-        if (isspace(*q)) {
+        if (nasm_isspace(*q)) {
             *q++ = '\0';
-            while (*q && isspace(*q))
+            while (*q && nasm_isspace(*q))
                 q++;
         }
 
@@ -1679,20 +1682,20 @@ static int obj_directive(char *directive, char *value, int pass)
         if (pass == 2)
             return 1;           /* ignore in pass two */
         intname = q = value;
-        while (*q && !isspace(*q))
+        while (*q && !nasm_isspace(*q))
             q++;
-        if (isspace(*q)) {
+        if (nasm_isspace(*q)) {
             *q++ = '\0';
-            while (*q && isspace(*q))
+            while (*q && nasm_isspace(*q))
                 q++;
         }
 
         extname = q;
-        while (*q && !isspace(*q))
+        while (*q && !nasm_isspace(*q))
             q++;
-        if (isspace(*q)) {
+        if (nasm_isspace(*q)) {
             *q++ = '\0';
-            while (*q && isspace(*q))
+            while (*q && nasm_isspace(*q))
                 q++;
         }
 
@@ -1706,11 +1709,11 @@ static int obj_directive(char *directive, char *value, int pass)
         }
         while (*q) {
             v = q;
-            while (*q && !isspace(*q))
+            while (*q && !nasm_isspace(*q))
                 q++;
-            if (isspace(*q)) {
+            if (nasm_isspace(*q)) {
                 *q++ = '\0';
-                while (*q && isspace(*q))
+                while (*q && nasm_isspace(*q))
                     q++;
             }
             if (!nasm_stricmp(v, "resident"))
@@ -2292,24 +2295,7 @@ static void obj_fwrite(ObjRecord * orp)
     fputc((-cksum) & 0xFF, ofp);
 }
 
-static const char *obj_stdmac[] = {
-    "%define __SECT__ [section .text]",
-    "%imacro group 1+.nolist",
-    "[group %1]",
-    "%endmacro",
-    "%imacro uppercase 0+.nolist",
-    "[uppercase %1]",
-    "%endmacro",
-    "%imacro export 1+.nolist",
-    "[export %1]",
-    "%endmacro",
-    "%imacro import 1+.nolist",
-    "[import %1]",
-    "%endmacro",
-    "%macro __NASM_CDecl__ 1",
-    "%endmacro",
-    NULL
-};
+extern macros_t obj_stdmac[];
 
 void dbgbi_init(struct ofmt *of, void *id, FILE * fp, efunc error)
 {
