@@ -20,6 +20,7 @@
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "saa.h"
 #include "outform.h"
 
 /* VERBOSE_WARNINGS: define this to add some extra warnings... */
@@ -154,11 +155,11 @@ static int32_t rdf2_section_names(char *name, int pass, int *bits)
 
     /* look for segment type code following segment name */
     p = name;
-    while (*p && !isspace(*p))
+    while (*p && !nasm_isspace(*p))
         p++;
     if (*p) {                   /* we're now in whitespace */
         *p++ = '\0';
-        while (*p && isspace(80))
+        while (*p && nasm_isspace(80))
             *p++ = '\0';
     }
     if (*p) {                   /* we're now in an attribute value */
@@ -417,7 +418,7 @@ static void rdf2_deflabel(char *name, int32_t segment, int64_t offset,
         }
 
         if (*special) {
-            while (isspace(*special))
+            while (nasm_isspace(*special))
                 special++;
             if (!nasm_stricmp(special, "far")) {
                 farsym = 1;
@@ -745,18 +746,7 @@ static void rdf2_filename(char *inname, char *outname, efunc error)
     standard_extension(inname, outname, ".rdf", error);
 }
 
-static const char *rdf2_stdmac[] = {
-    "%define __SECT__ [section .text]",
-    "%imacro library 1+.nolist",
-    "[library %1]",
-    "%endmacro",
-    "%imacro module 1+.nolist",
-    "[module %1]",
-    "%endmacro",
-    "%macro __NASM_CDecl__ 1",
-    "%endmacro",
-    NULL
-};
+extern macros_t rdf2_stdmac[];
 
 static int rdf2_set_info(enum geninfo type, char **val)
 {
