@@ -24,6 +24,8 @@
  * under DOS. '#define STINGY_MEMORY' may help a little.
  */
 
+#include "compiler.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -45,9 +47,9 @@
  */
 
 struct segment_infonode {
-    int dest_seg;               /* output segment to be placed into, -1 to 
+    int dest_seg;               /* output segment to be placed into, -1 to
                                    skip linking this segment */
-    long reloc;                 /* segment's relocation factor */
+    int32_t reloc;                 /* segment's relocation factor */
 };
 
 struct modulenode {
@@ -57,7 +59,7 @@ struct modulenode {
     void *header;
     char *name;
     struct modulenode *next;
-    long bss_reloc;
+    int32_t bss_reloc;
 };
 
 #include "ldsegs.h"
@@ -70,10 +72,10 @@ struct modulenode {
  */
 
 void processmodule(const char *filename, struct modulenode *mod);
-int allocnewseg(uint16 type, uint16 reserved);
-int findsegment(uint16 type, uint16 reserved);
-void symtab_add(const char *symbol, int segment, long offset);
-int symtab_get(const char *symbol, int *segment, long *offset);
+int allocnewseg(uint16_t type, uint16_t reserved);
+int findsegment(uint16_t type, uint16_t reserved);
+void symtab_add(const char *symbol, int segment, int32_t offset);
+int symtab_get(const char *symbol, int *segment, int32_t *offset);
 
 /* =========================================================================
  * Global data structures.
@@ -107,11 +109,11 @@ rdf_headerbuf *newheader = NULL;
 
 /* The current state of segment allocation, including information about
  * which output segment numbers have been allocated, and their types and
- * amount of data which has already been allocated inside them. 
+ * amount of data which has already been allocated inside them.
  */
 struct SegmentHeaderRec outputseg[RDF_MAXSEGS];
 int nsegs = 0;
-long bss_length;
+int32_t bss_length;
 
 /* global options which affect how the program behaves */
 struct ldrdfoptions {
@@ -186,7 +188,7 @@ void loadmodule(const char *filename)
         exit(1);
     }
 
-    /* 
+    /*
      * store information about the module, and determine what segments
      * it contains, and what we should do with them (determine relocation
      * factor if we decide to keep them)
@@ -213,9 +215,11 @@ void processmodule(const char *filename, struct modulenode *mod)
     int seg, outseg;
     void *header;
     rdfheaderrec *hr;
-    long bssamount = 0;
+    int32_t bssamount = 0;
     int bss_was_referenced = 0;
 
+    memset(&sconf, 0, sizeof sconf);
+
     for (seg = 0; seg < mod->f.nsegs; seg++) {
         /*
          * get the segment configuration for this type from the segment
@@ -254,7 +258,7 @@ void processmodule(const char *filename, struct modulenode *mod)
             mod->seginfo[seg].reloc = 0;
             outputseg[outseg].length = mod->f.seg[seg].length;
             if (options.verbose > 1)
-                printf("=> %04x:%08lx (+%04lx)\n", outseg,
+                printf("=> %04x:%08"PRIx32" (+%04"PRIx32")\n", outseg,
                        mod->seginfo[seg].reloc, mod->f.seg[seg].length);
             break;
 
@@ -281,7 +285,7 @@ void processmodule(const char *filename, struct modulenode *mod)
             outputseg[outseg].length += mod->f.seg[seg].length;
 
             if (options.verbose > 1)
-                printf("=> %04x:%08lx (+%04lx)\n", outseg,
+                printf("=> %04x:%08"PRIx32" (+%04"PRIx32")\n", outseg,
                        mod->seginfo[seg].reloc, mod->f.seg[seg].length);
         }
 
@@ -311,7 +315,7 @@ void processmodule(const char *filename, struct modulenode *mod)
 
         case RDFREC_GLOBAL:{   /* exported symbol */
                 int destseg;
-                long destreloc;
+                int32_t destreloc;
 
                 if (hr->e.segment == 2) {
                     bss_was_referenced = 1;
@@ -349,7 +353,7 @@ void processmodule(const char *filename, struct modulenode *mod)
                 if (bss_length % hr->c.align != 0)
                     bss_length += hr->c.align - (bss_length % hr->c.align);
                 if (options.verbose > 1) {
-                    printf("%s %04x common '%s' => 0002:%08lx (+%04lx)\n",
+                    printf("%s %04x common '%s' => 0002:%08"PRIx32" (+%04"PRIx32")\n",
                            filename, hr->c.segment, hr->c.label,
                            bss_length, hr->c.size);
                 }
@@ -374,7 +378,7 @@ void processmodule(const char *filename, struct modulenode *mod)
 
         mod->bss_reloc = bss_length;
         if (options.verbose > 1) {
-            printf("%s 0002 [            BSS] => 0002:%08lx (+%04lx)\n",
+            printf("%s 0002 [            BSS] => 0002:%08"PRIx32" (+%04"PRIx32")\n",
                    filename, bss_length, bssamount);
         }
         bss_length += bssamount;
@@ -417,7 +421,7 @@ int lookformodule(const char *name)
  * a segment of the type requested, and if one isn't found allocates a
  * new one.
  */
-int allocnewseg(uint16 type, uint16 reserved)
+int allocnewseg(uint16_t type, uint16_t reserved)
 {
     outputseg[nsegs].type = type;
     outputseg[nsegs].number = nsegs;
@@ -429,7 +433,7 @@ int allocnewseg(uint16 type, uint16 reserved)
     return nsegs++;
 }
 
-int findsegment(uint16 type, uint16 reserved)
+int findsegment(uint16_t type, uint16_t reserved)
 {
     int i;
 
@@ -457,7 +461,7 @@ int findsegment(uint16 type, uint16 reserved)
  * routine won't change a previously existing symbol. It will change
  * to segment = -2 only if the segment was previously < 0.
  */
-void symtab_add(const char *symbol, int segment, long offset)
+void symtab_add(const char *symbol, int segment, int32_t offset)
 {
     symtabEnt *ste;
 
@@ -509,7 +513,7 @@ void symtab_add(const char *symbol, int segment, long offset)
  * are assumed to have -1:0 associated. Returns 1 if the symbol was
  * successfully located.
  */
-int symtab_get(const char *symbol, int *segment, long *offset)
+int symtab_get(const char *symbol, int *segment, int32_t *offset)
 {
     symtabEnt *ste = symtabFind(symtab, symbol);
     if (!ste) {
@@ -575,7 +579,7 @@ int search_libraries()
     int i;
     void *header;
     int segment;
-    long offset;
+    int32_t offset;
     int doneanything = 0, pass = 1, keepfile;
     rdfheaderrec *hr;
 
@@ -685,8 +689,8 @@ void write_output(const char *filename)
     rdfheaderrec *hr, newrec;
     symtabEnt *se;
     segtab segs;
-    long offset;
-    byte *data;
+    int32_t offset;
+    uint8_t *data;
 
     if ((f = fopen(filename, "wb")) == NULL) {
         fprintf(stderr, "ldrdf: couldn't open %s for output\n", filename);
@@ -718,10 +722,11 @@ void write_output(const char *filename)
         }
         i = fread(hr->g.data, 1, sizeof(hr->g.data), ff);
         fseek(ff, 0, SEEK_END);
-        if (ftell(ff) > sizeof(hr->g.data)) {
+        if (ftell(ff) > (long)sizeof(hr->g.data)) {
             fprintf(error_file,
-                    "warning: maximum generic record size is %d, rest of file ignored\n",
-                    sizeof(hr->g.data));
+                    "warning: maximum generic record size is %u, "
+		    "rest of file ignored\n",
+                    (unsigned int)sizeof(hr->g.data));
         }
         fclose(ff);
 
@@ -857,7 +862,7 @@ void write_output(const char *filename)
                     break;
                 }
 
-                /* 
+                /*
                  * okay, now the relocation is in the segment pointed to by
                  * cur->seginfo[localseg], and we know everything else is
                  * okay to go ahead and do the relocation
@@ -883,21 +888,21 @@ void write_output(const char *filename)
                     if (offset < -127 || offset > 128)
                         fprintf(error_file,
                                 "warning: relocation out of range "
-                                "at %s(%02x:%08lx)\n", cur->name,
+                                "at %s(%02x:%08"PRIx32")\n", cur->name,
                                 (int)hr->r.segment, hr->r.offset);
                     *data = (char)offset;
                     break;
                 case 2:
-                    offset += *(short *)data;
+                    offset += *(int16_t *)data;
                     if (offset < -32767 || offset > 32768)
                         fprintf(error_file,
                                 "warning: relocation out of range "
-                                "at %s(%02x:%08lx)\n", cur->name,
+                                "at %s(%02x:%08"PRIx32")\n", cur->name,
                                 (int)hr->r.segment, hr->r.offset);
-                    *(short *)data = (short)offset;
+                    *(int16_t *)data = (int16_t)offset;
                     break;
                 case 4:
-                    *(long *)data += offset;
+                    *(int32_t *)data += offset;
                     /* we can't easily detect overflow on this one */
                     break;
                 }
@@ -923,7 +928,7 @@ void write_output(const char *filename)
             case RDFREC_FARIMPORT:
                 /*
                  * scan the global symbol table for the symbol
-                 * and associate its location with the segment number 
+                 * and associate its location with the segment number
                  * for this module
                  */
                 se = symtabFind(symtab, hr->i.label);
@@ -1096,19 +1101,19 @@ void write_output(const char *filename)
      * the output file
      */
     for (i = 0; i < nsegs; i++) {
-        uint16 s;
-        long l;
+        uint16_t s;
+        int32_t l;
 
         if (i == 2)
             continue;
 
-        s = translateshort(outputseg[i].type);
+        s = translateint16_t(outputseg[i].type);
         fwrite(&s, 2, 1, f);
-        s = translateshort(outputseg[i].number);
+        s = translateint16_t(outputseg[i].number);
         fwrite(&s, 2, 1, f);
-        s = translateshort(outputseg[i].reserved);
+        s = translateint16_t(outputseg[i].reserved);
         fwrite(&s, 2, 1, f);
-        l = translatelong(outputseg[i].length);
+        l = translateint32_t(outputseg[i].length);
         fwrite(&l, 4, 1, f);
 
         fwrite(outputseg[i].data, outputseg[i].length, 1, f);
