@@ -2,6 +2,9 @@
  * rdlib.h	Functions for manipulating libraries of RDOFF object files.
  */
 
+#ifndef RDOFF_RDLIB_H
+#define RDOFF_RDLIB_H 1
+
 struct librarynode {
     char *name;
     FILE *fp;                   /* initialised to NULL - always check */
@@ -22,3 +25,5 @@ int rdl_searchlib(struct librarynode *lib, const char *label, rdffile * f);
 int rdl_openmodule(struct librarynode *lib, int module, rdffile * f);
 
 void rdl_perror(const char *apname, const char *filename);
+
+#endif
