@@ -1,3 +1,5 @@
+#include "compiler.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include "segtab.h"
@@ -5,11 +7,11 @@
 struct segtabnode {
     int localseg;
     int destseg;
-    long offset;
+    int32_t offset;
 
     struct segtabnode *left;
     struct segtabnode *right;
-    /* 
+    /*
      * counts of how many are left or right, for use in reorganising
      * the tree
      */
@@ -36,7 +38,7 @@ void init_seglocations(segtab * root)
 }
 
 void descend_tree_add(struct segtabnode **node,
-                      int localseg, int destseg, long offset)
+                      int localseg, int destseg, int32_t offset)
 {
     struct segtabnode *n;
 
@@ -83,14 +85,14 @@ void descend_tree_add(struct segtabnode **node,
     }
 }
 
-void add_seglocation(segtab * root, int localseg, int destseg, long offset)
+void add_seglocation(segtab * root, int localseg, int destseg, int32_t offset)
 {
     descend_tree_add((struct segtabnode **)root, localseg, destseg,
                      offset);
 }
 
 int get_seglocation(segtab * root, int localseg, int *destseg,
-                    long *offset)
+                    int32_t *offset)
 {
     struct segtabnode *n = (struct segtabnode *)*root;
 
